@@ -3,6 +3,7 @@
  *
  * Copyright (C) 1999 Dave Cole
  * Copyright (C) 2003,2006 Bas Wijnen <shevek@fmf.nl>
+ * Copyright (C) 2011 Roland Clobus <rclobus@rclobus.nl>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +24,7 @@
 #include "version.h"
 #include "game.h"
 #include "ai.h"
+#include "client.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
@@ -39,11 +41,13 @@ static gboolean enable_debug = FALSE;
 static gboolean show_version = FALSE;
 static Map *map = NULL;
 
+static void logbot_init(void);
+
 static const struct algorithm_info {
 	/** Name of the algorithm (for commandline) */
 	const gchar *name;
 	/** Init function */
-	void (*init_func) (int argc, char **argv);
+	void (*init_func) (void);
 	/** Request to be a player? */
 	gboolean request_player;
 	/** Quit if request not honoured */
@@ -52,6 +56,7 @@ static const struct algorithm_info {
 /* *INDENT-OFF* */
 	{ "greedy", &greedy_init, TRUE, TRUE},
 	{ "lobbybot", &lobbybot_init, FALSE, TRUE},
+	{ "logbot", &logbot_init, FALSE, TRUE},
 /* *INDENT-ON* */
 };
 
@@ -87,17 +92,19 @@ static GOptionEntry commandline_entries[] = {
 	{NULL, '\0', 0, 0, NULL, NULL, NULL}
 };
 
-static void ai_init(int argc, char **argv)
+static void ai_init_glib_et_al(int argc, char **argv)
 {
 	GOptionContext *context;
 	GError *error = NULL;
 
-	/* Long description in the commandline for pioneersai: help */
 	context =
+	    /* Long description in the commandline for pioneersai: help */
 	    g_option_context_new(_("- Computer player for Pioneers"));
 	g_option_context_add_main_entries(context, commandline_entries,
 					  PACKAGE);
 	g_option_context_parse(context, &argc, &argv, &error);
+	g_option_context_free(context);
+
 	if (error != NULL) {
 		g_print("%s\n", error->message);
 		g_error_free(error);
@@ -111,6 +118,13 @@ static void ai_init(int argc, char **argv)
 		exit(0);
 	}
 
+	g_type_init();
+	set_ui_driver(&Glib_Driver);
+	log_set_func_default();
+}
+
+static void ai_init(void)
+{
 	set_enable_debug(enable_debug);
 
 	if (server == NULL)
@@ -128,9 +142,6 @@ static void ai_init(int argc, char **argv)
 		exit(0);
 	}
 
-	set_ui_driver(&Glib_Driver);
-	log_set_func_default();
-
 	if (ai != NULL) {
 		gint i;
 		for (i = 0; i < G_N_ELEMENTS(algorithms); i++) {
@@ -140,7 +151,7 @@ static void ai_init(int argc, char **argv)
 	}
 	log_message(MSG_INFO, _("Type of computer player: %s\n"),
 		    algorithms[active_algorithm].name);
-	algorithms[active_algorithm].init_func(argc, argv);
+	algorithms[active_algorithm].init_func();
 }
 
 void ai_panic(const char *message)
@@ -154,10 +165,12 @@ static void ai_offline(void)
 	gchar *style;
 
 	callbacks.offline = callbacks.quit;
+	notifying_string_set(requested_name, name);
 	style =
 	    g_strdup_printf("ai %s", algorithms[active_algorithm].name);
-	cb_connect(server, port, name,
-		   !algorithms[active_algorithm].request_player, style);
+	notifying_string_set(requested_style, style);
+	cb_connect(server, port,
+		   !algorithms[active_algorithm].request_player);
 	g_free(style);
 	g_free(name);
 }
@@ -165,7 +178,7 @@ static void ai_offline(void)
 static void ai_start_game(void)
 {
 	if (algorithms[active_algorithm].request_player ==
-	    my_player_viewer()
+	    my_player_spectator()
 	    && algorithms[active_algorithm].quit_if_not_request) {
 		ai_panic(N_("The game is already full. I'm leaving."));
 	}
@@ -194,9 +207,17 @@ static void ai_set_map(Map * new_map)
 
 void frontend_set_callbacks(void)
 {
+	callbacks.init_glib_et_al = &ai_init_glib_et_al;
 	callbacks.init = &ai_init;
 	callbacks.offline = &ai_offline;
 	callbacks.start_game = &ai_start_game;
 	callbacks.get_map = &ai_get_map;
 	callbacks.set_map = &ai_set_map;
 }
+
+/* The logbot is intended to be used as a spectator in a game, and to collect
+ * a transcript of the game in human readable form, which can be analysed
+ * using external tools. */
+void logbot_init(void)
+{
+}
