@@ -33,6 +33,7 @@
 #include "callback.h"
 #include "buildrec.h"
 #include "quoteinfo.h"
+#include "notifying-string.h"
 
 static enum callback_mode previous_mode;
 GameParams *game_params;
@@ -49,6 +50,10 @@ static struct recovery_info_t {
 	gboolean ship_moved;
 } recovery_info;
 
+NotifyingString *requested_name = NULL;
+NotifyingString *requested_style = NULL;
+gboolean requested_spectator;
+
 static gboolean global_unhandled(StateMachine * sm, gint event);
 static gboolean global_filter(StateMachine * sm, gint event);
 static gboolean mode_offline(StateMachine * sm, gint event);
@@ -107,7 +112,12 @@ static void waiting_for_network(gboolean is_waiting)
 }
 
 /* Dummy callback functions. They do nothing */
-static void dummy_init(G_GNUC_UNUSED int argc, G_GNUC_UNUSED char **argv)
+static void dummy_init_glib_et_al(G_GNUC_UNUSED int argc,
+				  G_GNUC_UNUSED char **argv)
+{;
+}
+
+static void dummy_init(void)
 {;
 }
 
@@ -363,8 +373,8 @@ static void dummy_new_points(G_GNUC_UNUSED gint player_num,
 {
 }
 
-static void dummy_viewer_name(G_GNUC_UNUSED gint viewer_num,
-			      G_GNUC_UNUSED const gchar * name)
+static void dummy_spectator_name(G_GNUC_UNUSED gint spectator_num,
+				 G_GNUC_UNUSED const gchar * name)
 {;
 }
 
@@ -382,7 +392,7 @@ static void dummy_player_quit(G_GNUC_UNUSED gint player_num)
 {;
 }
 
-static void dummy_viewer_quit(G_GNUC_UNUSED gint player_num)
+static void dummy_spectator_quit(G_GNUC_UNUSED gint player_num)
 {;
 }
 
@@ -417,6 +427,7 @@ void client_init(void)
 	 * someone forgets to update this when adding a new callback */
 	memset(&callbacks, 0, sizeof(callbacks));
 	/* set all callbacks to their default value: doing nothing */
+	callbacks.init_glib_et_al = &dummy_init_glib_et_al;
 	callbacks.init = &dummy_init;
 	callbacks.network_status = &dummy_network_status;
 	callbacks.instructions = &dummy_instructions;
@@ -471,11 +482,11 @@ void client_init(void)
 	callbacks.get_rolled_resources = &dummy_get_rolled_resources;
 	callbacks.new_statistics = &dummy_new_statistics;
 	callbacks.new_points = &dummy_new_points;
-	callbacks.viewer_name = &dummy_viewer_name;
+	callbacks.spectator_name = &dummy_spectator_name;
 	callbacks.player_name = &dummy_player_name;
 	callbacks.player_style = &dummy_player_style;
 	callbacks.player_quit = &dummy_player_quit;
-	callbacks.viewer_quit = &dummy_viewer_quit;
+	callbacks.spectator_quit = &dummy_spectator_quit;
 	callbacks.incoming_chat = &dummy_incoming_chat;
 	callbacks.new_bank = &dummy_new_bank;
 	callbacks.error = &dummy_error;
@@ -487,7 +498,12 @@ void client_init(void)
 
 void client_start(int argc, char **argv)
 {
-	callbacks.init(argc, argv);
+	callbacks.init_glib_et_al(argc, argv);
+
+	requested_name = NOTIFYING_STRING(notifying_string_new());
+	requested_style = NOTIFYING_STRING(notifying_string_new());
+
+	callbacks.init();
 	sm_goto(SM(), mode_offline);
 }
 
@@ -633,7 +649,7 @@ static gboolean check_other_players(StateMachine * sm)
 	DevelType devel_type;
 	Resource resource_type, supply_type, receive_type;
 	gint player_num, victim_num, card_idx, backwards;
-	gint turn_num, discard_num, num, ratio, die1, die2, x, y, pos;
+	gint discard_num, num, ratio, die1, die2, x, y, pos;
 	gint id;
 	gint resource_list[NO_RESOURCE], wanted_list[NO_RESOURCE];
 	gint sx, sy, spos, dx, dy, dpos;
@@ -728,8 +744,8 @@ static gboolean check_other_players(StateMachine * sm)
 		develop_played(player_num, card_idx, devel_type);
 		return TRUE;
 	}
-	if (sm_recv(sm, "turn %d", &turn_num)) {
-		turn_begin(player_num, turn_num);
+	if (sm_recv(sm, "turn %d", &num)) {
+		turn_begin(player_num, num);
 		return TRUE;
 	}
 	if (sm_recv(sm, "rolled %d %d", &die1, &die2)) {
@@ -821,6 +837,9 @@ static gboolean check_other_players(StateMachine * sm)
 	}
 	if (sm_recv(sm, "won with %d", &num)) {
 		callbacks.game_over(player_num, num);
+		log_message(MSG_DICE, _("%s has won the game with %d "
+					"victory points!\n"),
+			    player_name(player_num, TRUE), num);
 		sm_pop_all_and_goto(sm, mode_game_over);
 		return TRUE;
 	}
@@ -929,38 +948,40 @@ gboolean mode_start(StateMachine * sm, gint event)
 		return TRUE;
 	}
 	if (sm_recv(sm, "status report")) {
-		if (requested_viewer) {
-			if (requested_name[0] != '\0') {
-				sm_send(sm, "status viewer %s\n",
-					requested_name);
+		gchar *name = notifying_string_get(requested_name);
+		if (requested_spectator) {
+			if (name && name[0] != '\0') {
+				sm_send(sm, "status viewer %s\n", name);
 			} else {
 				sm_send(sm, "status newviewer\n");
 			}
 		} else {
-			if (requested_name[0] != '\0') {
-				sm_send(sm, "status reconnect %s\n",
-					requested_name);
+			if (name && name[0] != '\0') {
+				sm_send(sm, "status reconnect %s\n", name);
 			} else {
 				sm_send(sm, "status newplayer\n");
 			}
 		}
+		g_free(name);
 		return TRUE;
 	}
 	if (sm_recv(sm, "player %d of %d, welcome to pioneers server %S",
 		    &player_num, &total_num, &version)) {
+		gchar *style = notifying_string_get(requested_style);
+
 		g_free(version);
 		player_set_my_num(player_num);
 		player_set_total_num(total_num);
-		sm_send(sm, "style %s\n", requested_style);
+		sm_send(sm, "style %s\n", style);
 		sm_send(sm, "players\n");
 		sm_goto(sm, mode_players);
-
+		g_free(style);
 		return TRUE;
 	}
 	if (sm_recv(sm, "ERR sorry, version conflict")) {
 		sm_pop_all_and_goto(sm, mode_offline);
 		callbacks.network_status(_("Offline"));
-		callbacks.instructions(_("Version mismatch"));
+		callbacks.instructions(_("Version mismatch."));
 		log_message(MSG_ERROR,
 			    _("Version mismatch. Please make sure client "
 			      "and server are up to date.\n"));
@@ -1177,8 +1198,11 @@ static gboolean mode_load_gameinfo(StateMachine * sm, gint event)
 	     &num_roads, &num_bridges, &num_ships, &num_settlements,
 	     &num_cities, &num_soldiers, &road_len, &pchapel, &puniv,
 	     &pgov, &plibr, &pmarket, &plongestroad, &plargestarmy)) {
-		player_modify_statistic(my_player_num(), STAT_SOLDIERS,
-					num_soldiers);
+		if (num_soldiers) {
+			player_modify_statistic(my_player_num(),
+						STAT_SOLDIERS,
+						num_soldiers);
+		}
 		if (pchapel) {
 			player_modify_statistic(my_player_num(),
 						STAT_CHAPEL, 1);
@@ -1435,7 +1459,7 @@ static char *setup_msg(void)
 			}
 		}
 		old = msg;
-		msg = g_strdup_printf("%s %s", msg, parts[idx]);
+		msg = g_strdup_printf("%s %s.", msg, parts[idx]);
 		g_free(old);
 	}
 
@@ -1483,8 +1507,11 @@ static gboolean mode_idle(StateMachine * sm, gint event)
 	switch (event) {
 	case SM_ENTER:
 		callback_mode = MODE_WAIT_TURN;
-		if (!player_is_viewer(my_player_num()))
-			callbacks.instructions(_("Waiting for your turn"));
+		if (player_is_spectator(my_player_num()))
+			callbacks.instructions("");
+		else
+			callbacks.instructions(_
+					       ("Waiting for your turn."));
 		break;
 	case SM_RECV:
 		if (sm_recv(sm, "setup %d", &backwards)) {
@@ -1617,7 +1644,6 @@ gboolean mode_robber_response(StateMachine * sm, gint event)
 	case SM_RECV:
 		if (sm_recv(sm, "robber-done")) {
 			waiting_for_network(FALSE);
-			sm_stack_dump(sm);
 			/* current state is response
 			 * parent is steal
 			 * parent is move_response
@@ -1643,7 +1669,7 @@ static gboolean mode_robber(StateMachine * sm, gint event)
 	switch (event) {
 	case SM_ENTER:
 		callback_mode = MODE_ROBBER;
-		callbacks.instructions(_("Place the robber"));
+		callbacks.instructions(_("Place the robber."));
 		robber_begin_move(my_player_num());
 		callbacks.robber();
 		break;
