@@ -35,21 +35,7 @@
 static gboolean played_develop;	/* already played a non-victory card? */
 static gboolean bought_develop;	/* have we bought a development card? */
 
-static struct {
-	const gchar *name;
-	gboolean is_unique;
-} devel_cards[] = {
-	{
-	N_("Road Building"), FALSE}, {
-	N_("Monopoly"), FALSE}, {
-	N_("Year of Plenty"), FALSE}, {
-	N_("Chapel"), TRUE}, {
-	N_("Pioneer University"), TRUE}, {
-	N_("Governor's House"), TRUE}, {
-	N_("Library"), TRUE}, {
-	N_("Market"), TRUE}, {
-	N_("Soldier"), FALSE}
-};
+static gboolean is_unique[NUM_DEVEL_TYPES];	/* is each card unique? */
 
 static DevelDeck *develop_deck;	/* our deck of development cards */
 
@@ -59,9 +45,8 @@ void develop_init(void)
 	if (develop_deck != NULL)
 		deck_free(develop_deck);
 	develop_deck = deck_new(game_params);
-	for (idx = 0; idx < G_N_ELEMENTS(devel_cards); idx++)
-		devel_cards[idx].is_unique =
-		    game_params->num_develop_type[idx] == 1;
+	for (idx = 0; idx < NUM_DEVEL_TYPES; idx++)
+		is_unique[idx] = game_params->num_develop_type[idx] == 1;
 }
 
 void develop_bought_card_turn(DevelType type, gint turnbought)
@@ -75,18 +60,18 @@ void develop_bought_card_turn(DevelType type, gint turnbought)
 		/* Only log if the cards is bought in the current turn.
 		 * This function is also called during reconnect
 		 */
-		if (devel_cards[type].is_unique)
+		if (is_unique[type])
 			log_message(MSG_DEVCARD,
 				    /* This development card is unique */
 				    _(""
 				      "You bought the %s development card.\n"),
-				    gettext(devel_cards[type].name));
+				    get_devel_name(type));
 		else
 			log_message(MSG_DEVCARD,
 				    /* This development card is not unique */
 				    _(""
 				      "You bought a %s development card.\n"),
-				    gettext(devel_cards[type].name));
+				    get_devel_name(type));
 	};
 	player_modify_statistic(my_player_num(), STAT_DEVELOPMENT, 1);
 	stock_use_develop();
@@ -124,16 +109,16 @@ void develop_played(gint player_num, gint card_idx, DevelType type)
 	}
 	callbacks.played_develop(player_num, card_idx, type);
 
-	if (devel_cards[type].is_unique)
+	if (is_unique[type])
 		log_message(MSG_DEVCARD,
 			    _("%s played the %s development card.\n"),
 			    player_name(player_num, TRUE),
-			    gettext(devel_cards[type].name));
+			    get_devel_name(type));
 	else
 		log_message(MSG_DEVCARD,
 			    _("%s played a %s development card.\n"),
 			    player_name(player_num, TRUE),
-			    gettext(devel_cards[type].name));
+			    get_devel_name(type));
 
 	player_modify_statistic(player_num, STAT_DEVELOPMENT, -1);
 	switch (type) {
@@ -182,20 +167,20 @@ void monopoly_player(gint player_num, gint victim_num, gint num,
 
 	buf = resource_cards(num, type);
 	if (player_num == my_player_num()) {
-		/* I get the cards!
-		 */
+		/* I get the cards */
+		/* $1=resources, $2=player that loses resources */
 		log_message(MSG_STEAL, _("You get %s from %s.\n"),
 			    buf, player_name(victim_num, FALSE));
 		resource_modify(type, num);
 	} else if (victim_num == my_player_num()) {
-		/* I lose the cards!
-		 */
+		/* I lose the cards */
+		/* $1=player that steals, $2=resources */
 		log_message(MSG_STEAL, _("%s took %s from you.\n"),
 			    player_name(player_num, TRUE), buf);
 		resource_modify(type, -num);
 	} else {
-		/* I am a bystander
-		 */
+		/* I'm a bystander */
+		/* $1=player that steals, $2=resources, $3=player that loses resources */
 		tmp = g_strdup(player_name(player_num, TRUE));
 		log_message(MSG_STEAL, _("%s took %s from %s.\n"),
 			    tmp, buf, player_name(victim_num, FALSE));
@@ -249,8 +234,3 @@ const DevelDeck *get_devel_deck(void)
 {
 	return develop_deck;
 }
-
-const gchar *get_devel_name(DevelType type)
-{
-	return gettext(devel_cards[type].name);
-}
