@@ -3,7 +3,7 @@
  *
  * Copyright (C) 1999 Dave Cole
  * Copyright (C) 2003,2006 Bas Wijnen <shevek@fmf.nl>
- * Copyright (C) 2004 Roland Clobus <rclobus@bigfoot.com>
+ * Copyright (C) 2004,2010 Roland Clobus <rclobus@rclobus.nl>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,22 +34,23 @@
 #include "game-settings.h"
 #include "game-rules.h"
 #include "metaserver.h"
+#include "avahi.h"
+#include "avahi-browser.h"
+#include "client.h"
+#include "gtkcompat.h"
 
 const int PRIVATE_GAME_HISTORY_SIZE = 10;
 
-static gchar *connect_name;	/* Name of the player */
-static gboolean connect_viewer;	/* Prefer to be a viewer */
-static gboolean connect_viewer_allowed;	/* Viewer allowed */
+static gboolean connect_spectator;	/* Prefer to be a spectator */
+static gboolean connect_spectator_allowed;	/* Spectator allowed */
 static gchar *connect_server;	/* Name of the server */
 static gchar *connect_port;	/* Port of the server */
-static gchar *connect_style = NULL;	/* Style of the player */
 
 static GtkWidget *connect_dlg;	/* Dialog for starting a new game */
 static GtkWidget *name_entry;	/* Name of the player */
-static GtkWidget *viewer_toggle;	/* Prefer to be a viewer */
+static GtkWidget *spectator_toggle;	/* Prefer to be a spectator */
 static GtkWidget *meta_server_entry;	/* Name of the metaserver */
 
-
 static GtkWidget *meta_dlg;	/* Dialog for joining a public game */
 static GtkWidget *server_status;	/* Description of the current metaserver */
 static GtkListStore *meta_games_model;	/* The list of the games at the metaserver */
@@ -63,6 +64,7 @@ enum {
 enum {				/* The columns of the meta_games_model */
 	C_META_HOST,
 	C_META_PORT,
+	C_META_HOST_SORTABLE,	/* Invisible, used for sorting */
 	C_META_VERSION,
 	C_META_MAX,
 	C_META_CUR,
@@ -149,7 +151,7 @@ static void close_waiting_box(void);
 
 static void connect_set_field(gchar ** field, const gchar * value);
 static void connect_close_all(gboolean user_pressed_ok,
-			      gboolean can_be_viewer);
+			      gboolean can_be_spectator);
 static void set_meta_serverinfo(void);
 static void connect_private_dialog(G_GNUC_UNUSED GtkWidget * widget,
 				   GtkWindow * parent);
@@ -163,32 +165,38 @@ static void connect_set_field(gchar ** field, const gchar * value)
 	g_free(temp);
 }
 
-/* Public functions */
-const gchar *connect_get_name(void)
+/* Reset all pointers to the destroyed children of the dialog */
+static void connect_dlg_destroyed(GtkWidget * widget,
+				  GtkWidget ** widget_pointer)
 {
-	return connect_name;
+	name_entry = NULL;
+	spectator_toggle = NULL;
+	meta_server_entry = NULL;
+	gtk_widget_destroyed(widget, widget_pointer);
 }
 
-void connect_set_name(const gchar * name)
+static void connect_name_change_cb(G_GNUC_UNUSED gpointer ns)
 {
-	connect_set_field(&connect_name, name);
+	gchar *name = notifying_string_get(requested_name);
 	if (name_entry != NULL)
-		gtk_entry_set_text(GTK_ENTRY(name_entry), connect_name);
+		gtk_entry_set_text(GTK_ENTRY(name_entry), name);
+	g_free(name);
 }
 
-gboolean connect_get_viewer(void)
+/* Public functions */
+gboolean connect_get_spectator(void)
 {
-	return connect_viewer && connect_viewer_allowed;
+	return connect_spectator && connect_spectator_allowed;
 }
 
-void connect_set_viewer(gboolean viewer)
+void connect_set_spectator(gboolean spectator)
 {
-	connect_viewer = viewer;
-	connect_viewer_allowed = TRUE;
-	if (viewer_toggle != NULL)
+	connect_spectator = spectator;
+	connect_spectator_allowed = TRUE;
+	if (spectator_toggle != NULL)
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
-					     (viewer_toggle),
-					     connect_viewer);
+					     (spectator_toggle),
+					     connect_spectator);
 }
 
 const gchar *connect_get_server(void)
@@ -226,36 +234,23 @@ void connect_set_port(const gchar * port)
 	connect_set_field(&connect_port, port);
 }
 
-const gchar *connect_get_style(void)
-{
-	return connect_style;
-}
-
-void connect_set_style(const gchar * style)
-{
-	g_free(connect_style);
-	connect_style = g_strdup(style);
-}
-
 static void connect_close_all(gboolean user_pressed_ok,
-			      gboolean can_be_viewer)
+			      gboolean can_be_spectator)
 {
-	connect_viewer_allowed = can_be_viewer;
+	connect_spectator_allowed = can_be_spectator;
 	if (user_pressed_ok) {
 		gchar *meta_server;
 
-		connect_set_field(&connect_name,
-				  gtk_entry_get_text(GTK_ENTRY
-						     (name_entry)));
-		connect_viewer =
+		notifying_string_set(requested_name,
+				     gtk_entry_get_text(GTK_ENTRY
+							(name_entry)));
+		connect_spectator =
 		    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
-						 (viewer_toggle));
+						 (spectator_toggle));
 		/* Save connect dialogue entries */
 		meta_server = connect_get_meta_server();
 		config_set_string("connect/meta-server", meta_server);
 		g_free(meta_server);
-		config_set_string("connect/name", connect_name);
-		config_set_int("connect/viewer", connect_viewer);
 
 		frontend_gui_register_destroy(connect_dlg,
 					      GUI_CONNECT_TRY);
@@ -427,13 +422,16 @@ static void server_end(void)
 		gtk_list_store_set(meta_games_model, &iter,
 				   C_META_HOST, server_host,
 				   C_META_PORT, server_port,
+				   C_META_HOST_SORTABLE,
+				   g_strdup_printf("%s:%s", server_host,
+						   server_port),
 				   C_META_VERSION, server_version,
-				   C_META_MAX, server_max,
-				   C_META_CUR, server_curr,
-				   C_META_TERRAIN, server_terrain,
-				   C_META_VICTORY, server_vpoints,
-				   C_META_SEVENS, server_sevenrule,
-				   C_META_MAP, server_title, -1);
+				   C_META_MAX, server_max, C_META_CUR,
+				   server_curr, C_META_TERRAIN,
+				   server_terrain, C_META_VICTORY,
+				   server_vpoints, C_META_SEVENS,
+				   server_sevenrule, C_META_MAP,
+				   server_title, -1);
 	}
 }
 
@@ -723,7 +721,8 @@ static GtkWidget *build_create_interface(void)
 			 G_CALLBACK(player_change_cb), NULL);
 
 	row = GTK_TABLE(game_settings)->nrows;
-	label = gtk_label_new(_("Number of AI Players"));
+	/* Label */
+	label = gtk_label_new(_("Number of computer players"));
 	gtk_widget_show(label);
 	gtk_table_attach(GTK_TABLE(game_settings), label, 0, 1, row,
 			 row + 1, GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
@@ -742,7 +741,8 @@ static GtkWidget *build_create_interface(void)
 			 1, 2, row, row + 1,
 			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 	gtk_widget_set_tooltip_text(aiplayers_spin,
-				    _("The number of AI players"));
+				    /* Tooltip */
+				    _("The number of computer players"));
 
 	game_rules = game_rules_new_metaserver();
 	gtk_widget_show(game_rules);
@@ -827,7 +827,8 @@ static void create_server_dlg(G_GNUC_UNUSED GtkWidget * widget,
 	set_meta_serverinfo();
 
 	cserver_dlg =
-	    gtk_dialog_new_with_buttons(_("Create a public game"), parent,
+	    /* Dialog caption */
+	    gtk_dialog_new_with_buttons(_("Create a Public Game"), parent,
 					GTK_DIALOG_DESTROY_WITH_PARENT,
 					GTK_STOCK_CANCEL,
 					GTK_RESPONSE_CANCEL, GTK_STOCK_OK,
@@ -949,12 +950,14 @@ static void create_meta_dlg(G_GNUC_UNUSED GtkWidget * widget,
 		return;
 	}
 
-	meta_dlg = gtk_dialog_new_with_buttons(_("Join a public game"),
+	/* Dialog caption */
+	meta_dlg = gtk_dialog_new_with_buttons(_("Join a Public Game"),
 					       GTK_WINDOW(parent),
 					       0,
 					       GTK_STOCK_REFRESH,
 					       META_RESPONSE_REFRESH,
-					       _("_New remote game"),
+					       /* Button text */
+					       _("_New Remote Game"),
 					       META_RESPONSE_NEW,
 					       GTK_STOCK_CANCEL,
 					       GTK_RESPONSE_CANCEL,
@@ -967,16 +970,20 @@ static void create_meta_dlg(G_GNUC_UNUSED GtkWidget * widget,
 	gtk_widget_realize(meta_dlg);
 	gtk_widget_set_tooltip_text(gui_get_dialog_button
 				    (GTK_DIALOG(meta_dlg), 0),
+				    /* Tooltip */
 				    _("Refresh the list of games"));
 	gtk_widget_set_tooltip_text(gui_get_dialog_button
 				    (GTK_DIALOG(meta_dlg), 1),
+				    /* Tooltip */
 				    _(""
 				      "Create a new public game at the meta server"));
 	gtk_widget_set_tooltip_text(gui_get_dialog_button
 				    (GTK_DIALOG(meta_dlg), 2),
+				    /* Tooltip */
 				    _("Don't join a public game"));
 	gtk_widget_set_tooltip_text(gui_get_dialog_button
 				    (GTK_DIALOG(meta_dlg), 3),
+				    /* Tooltip */
 				    _("Join the selected game"));
 
 	dlg_vbox = GTK_DIALOG(meta_dlg)->vbox;
@@ -1005,6 +1012,7 @@ static void create_meta_dlg(G_GNUC_UNUSED GtkWidget * widget,
 					      G_TYPE_STRING, G_TYPE_STRING,
 					      G_TYPE_STRING, G_TYPE_STRING,
 					      G_TYPE_STRING, G_TYPE_STRING,
+					      G_TYPE_STRING,
 					      G_TYPE_STRING);
 
 	meta_games_view =
@@ -1012,9 +1020,11 @@ static void create_meta_dlg(G_GNUC_UNUSED GtkWidget * widget,
 	gtk_widget_show(meta_games_view);
 	gtk_container_add(GTK_CONTAINER(scroll_win), meta_games_view);
 	gtk_widget_set_tooltip_text(meta_games_view,
+				    /* Tooltip */
 				    _("Select a game to join"));
 
 	column =
+	    /* Column name */
 	    gtk_tree_view_column_new_with_attributes(_("Map Name"),
 						     gtk_cell_renderer_text_new
 						     (), "text",
@@ -1022,90 +1032,120 @@ static void create_meta_dlg(G_GNUC_UNUSED GtkWidget * widget,
 	gtk_tree_view_append_column(GTK_TREE_VIEW(meta_games_view),
 				    column);
 	gtk_tree_view_column_set_sort_column_id(column, C_META_MAP);
+	/* Tooltip for column 'Map Name' */
 	gtk_widget_set_tooltip_text(column->button, _("Name of the game"));
 
 	renderer = gtk_cell_renderer_text_new();
 	g_object_set(renderer, "xalign", 1.0f, NULL);
 	column =
+	    /* Column name */
 	    gtk_tree_view_column_new_with_attributes(_("Curr"), renderer,
 						     "text", C_META_CUR,
 						     NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(meta_games_view),
 				    column);
+	gtk_tree_view_column_set_sort_column_id(column, C_META_CUR);
 	gtk_widget_set_tooltip_text(column->button,
+				    /* Tooltip for column 'Curr' */
 				    _("Number of players in the game"));
 
 	renderer = gtk_cell_renderer_text_new();
 	g_object_set(renderer, "xalign", 1.0f, NULL);
 	column =
+	    /* Column name */
 	    gtk_tree_view_column_new_with_attributes(_("Max"), renderer,
 						     "text", C_META_MAX,
 						     NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(meta_games_view),
 				    column);
+	gtk_tree_view_column_set_sort_column_id(column, C_META_MAX);
 	gtk_widget_set_tooltip_text(column->button,
+				    /* Tooltip for column 'Max' */
 				    _("Maximum players for the game"));
 
 	column =
+	    /* Column name */
 	    gtk_tree_view_column_new_with_attributes(_("Terrain"),
 						     gtk_cell_renderer_text_new
 						     (), "text",
 						     C_META_TERRAIN, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(meta_games_view),
 				    column);
+	gtk_tree_view_column_set_sort_column_id(column, C_META_TERRAIN);
 	gtk_widget_set_tooltip_text(column->button,
+				    /* Tooltip for column 'Terrain' */
 				    _("Random of default terrain"));
 
 	renderer = gtk_cell_renderer_text_new();
 	g_object_set(renderer, "xalign", 1.0f, NULL);
 	column =
+	    /* Column name */
 	    gtk_tree_view_column_new_with_attributes(_("Vic. Points"),
 						     renderer, "text",
 						     C_META_VICTORY, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(meta_games_view),
 				    column);
+	gtk_tree_view_column_set_sort_column_id(column, C_META_VICTORY);
 	gtk_widget_set_tooltip_text(column->button,
+				    /* Tooltip for column 'Vic. Points' */
 				    _("Points needed to win"));
 
 	column =
+	    /* Column name */
 	    gtk_tree_view_column_new_with_attributes(_("Sevens Rule"),
 						     gtk_cell_renderer_text_new
 						     (), "text",
 						     C_META_SEVENS, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(meta_games_view),
 				    column);
+	gtk_tree_view_column_set_sort_column_id(column, C_META_SEVENS);
+	/* Tooltip for column 'Sevens Rule' */
 	gtk_widget_set_tooltip_text(column->button, _("Sevens rule"));
 
 	column =
+	    /* Column name */
 	    gtk_tree_view_column_new_with_attributes(_("Host"),
 						     gtk_cell_renderer_text_new
 						     (), "text",
 						     C_META_HOST, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(meta_games_view),
 				    column);
-	gtk_tree_view_column_set_sort_column_id(column, C_META_HOST);
+	gtk_tree_view_column_set_sort_column_id(column,
+						C_META_HOST_SORTABLE);
+	/* Tooltip for column 'Host' */
 	gtk_widget_set_tooltip_text(column->button, _("Host of the game"));
 
 	renderer = gtk_cell_renderer_text_new();
 	g_object_set(renderer, "xalign", 1.0f, NULL);
 	column =
+	    /* Column name */
 	    gtk_tree_view_column_new_with_attributes(_("Port"), renderer,
 						     "text", C_META_PORT,
 						     NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(meta_games_view),
 				    column);
+	gtk_tree_view_column_set_sort_column_id(column,
+						C_META_HOST_SORTABLE);
 	gtk_widget_set_tooltip_text(column->button,
-				    _("Port of the the game"));
+				    /* Tooltip for column 'Port' */
+				    _("Port of the game"));
 
 	column =
+	    /* Column name */
 	    gtk_tree_view_column_new_with_attributes(_("Version"),
 						     gtk_cell_renderer_text_new
 						     (), "text",
 						     C_META_VERSION, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(meta_games_view),
 				    column);
+	gtk_tree_view_column_set_sort_column_id(column, C_META_VERSION);
 	gtk_widget_set_tooltip_text(column->button,
+				    /* Tooltip for column 'Version' */
 				    _("Version of the host"));
+	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE
+					     (meta_games_model),
+					     C_META_HOST_SORTABLE,
+					     GTK_SORT_ASCENDING);
 
 	/* Register double-click */
 	g_signal_connect(G_OBJECT(meta_games_view), "button_press_event",
@@ -1142,6 +1182,26 @@ static void connect_dlg_cb(G_GNUC_UNUSED GtkDialog * dlg,
 	connect_close_all(FALSE, FALSE);
 }
 
+#ifdef HAVE_AVAHI
+static void connect_avahi_cb(G_GNUC_UNUSED GtkWidget * widget,
+			     GtkWidget * avahibrowser_entry)
+{
+	gchar *server =
+	    avahibrowser_get_server(AVAHIBROWSER(avahibrowser_entry));
+	gchar *port =
+	    avahibrowser_get_port(AVAHIBROWSER(avahibrowser_entry));
+
+	connect_set_field(&connect_server, server);
+	connect_set_field(&connect_port, port);
+
+	g_free(server);
+	g_free(port);
+
+	// connect
+	connect_close_all(TRUE, TRUE);
+}
+#endif
+
 void connect_create_dlg(void)
 {
 	GtkWidget *dlg_vbox;
@@ -1150,14 +1210,23 @@ void connect_create_dlg(void)
 	GtkWidget *hbox;
 	GtkWidget *btn;
 	GtkWidget *sep;
+#ifdef HAVE_AVAHI
+	GtkWidget *avahibrowser_entry;
+#endif				// HAVE_AVAHI
 	gchar *fullname;
+	gchar *name;
+	gint row;
 
 	if (connect_dlg) {
 		gtk_window_present(GTK_WINDOW(connect_dlg));
 		return;
 	}
 
-	connect_dlg = gtk_dialog_new_with_buttons(_("Start a new game"),
+	g_signal_connect(requested_name, "changed",
+			 G_CALLBACK(connect_name_change_cb), NULL);
+
+	/* Dialog caption */
+	connect_dlg = gtk_dialog_new_with_buttons(_("Start a New Game"),
 						  GTK_WINDOW(app_window),
 						  0,
 						  GTK_STOCK_CANCEL,
@@ -1166,7 +1235,7 @@ void connect_create_dlg(void)
 	g_signal_connect(G_OBJECT(connect_dlg), "response",
 			 G_CALLBACK(connect_dlg_cb), NULL);
 	g_signal_connect(G_OBJECT(connect_dlg), "destroy",
-			 G_CALLBACK(gtk_widget_destroyed), &connect_dlg);
+			 G_CALLBACK(connect_dlg_destroyed), &connect_dlg);
 
 	gtk_widget_realize(connect_dlg);
 	gdk_window_set_functions(connect_dlg->window,
@@ -1176,74 +1245,129 @@ void connect_create_dlg(void)
 	gtk_widget_show(dlg_vbox);
 
 	table = gtk_table_new(4, 3, FALSE);
+	row = 0;
 	gtk_widget_show(table);
 	gtk_box_pack_start(GTK_BOX(dlg_vbox), table, FALSE, TRUE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(table), 5);
 	gtk_table_set_row_spacings(GTK_TABLE(table), 3);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 5);
 
-	lbl = gtk_label_new(_("Player Name"));
+	/* Label */
+	lbl = gtk_label_new(_("Player name"));
 	gtk_widget_show(lbl);
-	gtk_table_attach(GTK_TABLE(table), lbl, 0, 1, 0, 1,
+	gtk_table_attach(GTK_TABLE(table), lbl, 0, 1, row, row + 1,
 			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 	gtk_misc_set_alignment(GTK_MISC(lbl), 0, 0.5);
 
+	name = notifying_string_get(requested_name);
 	name_entry = gtk_entry_new();
 	gtk_entry_set_max_length(GTK_ENTRY(name_entry), MAX_NAME_LENGTH);
 	gtk_widget_show(name_entry);
-	gtk_entry_set_text(GTK_ENTRY(name_entry), connect_name);
+	gtk_entry_set_text(GTK_ENTRY(name_entry), name);
+	g_free(name);
 
-	gtk_table_attach(GTK_TABLE(table), name_entry, 1, 2, 0, 1,
+	gtk_table_attach(GTK_TABLE(table), name_entry, 1, 2, row, row + 1,
 			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+	/* Tooltip */
 	gtk_widget_set_tooltip_text(name_entry, _("Enter your name"));
 
-	viewer_toggle = gtk_check_button_new_with_label(_("Viewer"));
-	gtk_widget_show(viewer_toggle);
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(viewer_toggle),
-				     connect_viewer);
+	/* Check button */
+	spectator_toggle = gtk_check_button_new_with_label(_("Spectator"));
+	gtk_widget_show(spectator_toggle);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(spectator_toggle),
+				     connect_spectator);
 
-	gtk_table_attach(GTK_TABLE(table), viewer_toggle, 2, 3, 0, 1, 0,
-			 GTK_EXPAND | GTK_FILL, 0, 0);
-	gtk_widget_set_tooltip_text(viewer_toggle,
-				    _("Do you want to be a viewer?"));
+	gtk_table_attach(GTK_TABLE(table), spectator_toggle, 2, 3, row,
+			 row + 1, 0, GTK_EXPAND | GTK_FILL, 0, 0);
+	gtk_widget_set_tooltip_text(spectator_toggle,
+				    /* Tooltip for checkbox Spectator */
+				    _
+				    ("Check if you want to be a spectator"));
+	row++;
 
 	sep = gtk_hseparator_new();
 	gtk_widget_show(sep);
-	gtk_table_attach(GTK_TABLE(table), sep, 0, 3, 1, 2,
+	gtk_table_attach(GTK_TABLE(table), sep, 0, 3, row, row + 1,
 			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 6);
+	row++;
 
-	lbl = gtk_label_new(_("Meta Server"));
+#ifdef HAVE_AVAHI
+	/* Label */
+	lbl = gtk_label_new(_("Avahi"));
 	gtk_widget_show(lbl);
-	gtk_table_attach(GTK_TABLE(table), lbl, 0, 1, 2, 3,
-			 (GtkAttachOptions) GTK_FILL,
-			 (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
+	gtk_table_attach(GTK_TABLE(table), lbl, 0, 1, row, row + 1,
+			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+	gtk_misc_set_alignment(GTK_MISC(lbl), 0, 0.5);
+
+	/* Button */
+	btn = gtk_button_new_with_label(_("Join"));
+	gtk_widget_show(btn);
+	gtk_widget_set_tooltip_text(btn,
+				    /* Tooltip for button Join */
+				    _(""
+				      "Join an automatically discovered game"));
+	gtk_table_attach(GTK_TABLE(table), btn, 2, 3, row, row + 1,
+			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+
+	avahibrowser_entry = avahibrowser_new(btn);
+	gtk_widget_show(avahibrowser_entry);
+	gtk_table_attach(GTK_TABLE(table), avahibrowser_entry, 1, 2, row,
+			 row + 1, GTK_EXPAND | GTK_FILL,
+			 GTK_EXPAND | GTK_FILL, 0, 0);
+	row++;
+
+	g_signal_connect(G_OBJECT(btn), "clicked",
+			 G_CALLBACK(connect_avahi_cb), avahibrowser_entry);
+
+	// enable avahi
+	// storing the pointer to this widget for later use
+	avahi_register(AVAHIBROWSER(avahibrowser_entry));
+
+	sep = gtk_hseparator_new();
+	gtk_widget_show(sep);
+	gtk_table_attach(GTK_TABLE(table), sep, 0, 3, row, row + 1,
+			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 6);
+	row++;
+#endif				// HAVE_AVAHI
+
+	/* Label */
+	lbl = gtk_label_new(_("Meta server"));
+	gtk_widget_show(lbl);
+	gtk_table_attach(GTK_TABLE(table), lbl, 0, 1, row, row + 1,
+			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 	gtk_misc_set_alignment(GTK_MISC(lbl), 0, 0.5);
 
 	meta_server_entry = metaserver_new();
 	gtk_widget_show(meta_server_entry);
-	gtk_table_attach(GTK_TABLE(table), meta_server_entry, 1, 3, 2, 3,
-			 (GtkAttachOptions) GTK_EXPAND | GTK_FILL,
-			 (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
+	gtk_table_attach(GTK_TABLE(table), meta_server_entry, 1, 3, row,
+			 row + 1, GTK_EXPAND | GTK_FILL,
+			 GTK_EXPAND | GTK_FILL, 0, 0);
 	metaserver_add(METASERVER(meta_server_entry),
 		       metaserver_info.server);
+	row++;
 
 	hbox = gtk_hbox_new(FALSE, 3);
 	gtk_widget_show(hbox);
-	gtk_table_attach(GTK_TABLE(table), hbox, 0, 3, 3, 4,
+	gtk_table_attach(GTK_TABLE(table), hbox, 0, 3, row, row + 1,
 			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 3);
+	row++;
 
-	btn = gtk_button_new_with_label(_("Join public game"));
+	/* Button */
+	btn = gtk_button_new_with_label(_("Join Public Game"));
 	gtk_widget_show(btn);
 	gtk_box_pack_start(GTK_BOX(hbox), btn, TRUE, TRUE, 0);
 	g_signal_connect(G_OBJECT(btn), "clicked",
 			 G_CALLBACK(create_meta_dlg), app_window);
+	/* Tooltip */
 	gtk_widget_set_tooltip_text(btn, _("Join a public game"));
-	GTK_WIDGET_SET_FLAGS(btn, GTK_CAN_DEFAULT);
+	gtk_widget_set_can_default(btn, TRUE);
 	gtk_widget_grab_default(btn);
 
-	btn = gtk_button_new_with_label(_("Create game"));
+	/* Button */
+	btn = gtk_button_new_with_label(_("Create Game"));
 	gtk_widget_show(btn);
 	gtk_box_pack_start(GTK_BOX(hbox), btn, TRUE, TRUE, 0);
+	/* Tooltip */
 	gtk_widget_set_tooltip_text(btn, _("Create a game"));
 	g_signal_connect(G_OBJECT(btn), "clicked",
 			 G_CALLBACK(launch_server_gtk), app_window);
@@ -1254,9 +1378,11 @@ void connect_create_dlg(void)
 		gtk_widget_set_sensitive(GTK_WIDGET(btn), FALSE);
 	}
 
-	btn = gtk_button_new_with_label(_("Join private game"));
+	/* Button */
+	btn = gtk_button_new_with_label(_("Join Private Game"));
 	gtk_widget_show(btn);
 	gtk_box_pack_start(GTK_BOX(hbox), btn, TRUE, TRUE, 0);
+	/* Tooltip */
 	gtk_widget_set_tooltip_text(btn, _("Join a private game"));
 	g_signal_connect(G_OBJECT(btn), "clicked",
 			 G_CALLBACK(connect_private_dialog), app_window);
@@ -1341,12 +1467,12 @@ static void update_recent_servers_list(void)
 static void host_list_select_cb(GtkWidget * widget, gpointer user_data)
 {
 	GPtrArray *host_entries = user_data;
-	gint index;
+	gint idx;
 	gchar *entry;
 	gchar **strs;
 
-	index = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
-	entry = g_ptr_array_index(host_entries, index);
+	idx = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
+	entry = g_ptr_array_index(host_entries, idx);
 	strs = g_strsplit(entry, ":", 2);
 
 	connect_set_field(&connect_server, strs[0]);
@@ -1400,12 +1526,18 @@ static void connect_private_dialog(G_GNUC_UNUSED GtkWidget * widget,
 		return;
 	}
 
-	connect_private_dlg =
-	    gtk_dialog_new_with_buttons(_("Join a private game"),
-					GTK_WINDOW(parent), 0,
-					GTK_STOCK_CANCEL,
-					GTK_RESPONSE_CANCEL, GTK_STOCK_OK,
-					GTK_RESPONSE_OK, NULL);
+	connect_private_dlg = gtk_dialog_new_with_buttons(
+								 /* Dialog caption */
+								 _(""
+								   "Join a private game"),
+								 GTK_WINDOW
+								 (parent),
+								 0,
+								 GTK_STOCK_CANCEL,
+								 GTK_RESPONSE_CANCEL,
+								 GTK_STOCK_OK,
+								 GTK_RESPONSE_OK,
+								 NULL);
 	gtk_dialog_set_default_response(GTK_DIALOG(connect_private_dlg),
 					GTK_RESPONSE_OK);
 	g_signal_connect(G_OBJECT(connect_private_dlg), "response",
@@ -1429,7 +1561,8 @@ static void connect_private_dialog(G_GNUC_UNUSED GtkWidget * widget,
 	gtk_table_set_row_spacings(GTK_TABLE(table), 3);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 5);
 
-	lbl = gtk_label_new(_("Server Host"));
+	/* Label */
+	lbl = gtk_label_new(_("Server host"));
 	gtk_widget_show(lbl);
 	gtk_table_attach(GTK_TABLE(table), lbl, 0, 1, 0, 1,
 			 GTK_FILL, GTK_EXPAND, 0, 0);
@@ -1442,10 +1575,12 @@ static void connect_private_dialog(G_GNUC_UNUSED GtkWidget * widget,
 			 GTK_EXPAND | GTK_FILL, 0, 0);
 	gtk_entry_set_text(GTK_ENTRY(host_entry), connect_server);
 	gtk_widget_set_tooltip_text(host_entry,
+				    /* Tooltip */
 				    _("Name of the host of the game"));
 	connect_set_field(&connect_server, connect_server);
 
-	lbl = gtk_label_new(_("Server Port"));
+	/* Label */
+	lbl = gtk_label_new(_("Server port"));
 	gtk_widget_show(lbl);
 	gtk_table_attach(GTK_TABLE(table), lbl, 0, 1, 1, 2,
 			 GTK_FILL, GTK_EXPAND, 0, 0);
@@ -1461,10 +1596,11 @@ static void connect_private_dialog(G_GNUC_UNUSED GtkWidget * widget,
 	gtk_box_pack_start(GTK_BOX(hbox), port_entry, FALSE, TRUE, 0);
 	gtk_entry_set_text(GTK_ENTRY(port_entry), connect_port);
 	gtk_widget_set_tooltip_text(port_entry,
+				    /* Tooltip */
 				    _("Port of the host of the game"));
 	connect_set_field(&connect_port, connect_port);
 
-	host_list = gtk_combo_box_new_text();
+	host_list = gtk_combo_box_text_new();
 	host_entries = g_ptr_array_new();
 
 	gtk_widget_show(host_list);
@@ -1491,8 +1627,9 @@ static void connect_private_dialog(G_GNUC_UNUSED GtkWidget * widget,
 		g_free(host_port);
 
 		g_ptr_array_add(host_entries, host_name_port);
-		gtk_combo_box_append_text(GTK_COMBO_BOX(host_list),
-					  host_name_port);
+		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT
+					       (host_list),
+					       host_name_port);
 	}
 	if (i > 0)
 		gtk_combo_box_set_active(GTK_COMBO_BOX(host_list), 0);
@@ -1501,9 +1638,11 @@ static void connect_private_dialog(G_GNUC_UNUSED GtkWidget * widget,
 
 	gtk_table_attach(GTK_TABLE(table), host_list, 1, 2, 2, 3,
 			 GTK_FILL, GTK_FILL, 0, 0);
+	/* Tooltip */
 	gtk_widget_set_tooltip_text(host_list, _("Recent games"));
 
-	lbl = gtk_label_new(_("Recent Games"));
+	/* Label */
+	lbl = gtk_label_new(_("Recent games"));
 	gtk_widget_show(lbl);
 	gtk_table_attach(GTK_TABLE(table), lbl, 0, 1, 2, 3,
 			 GTK_FILL, GTK_EXPAND, 0, 0);
