@@ -71,13 +71,18 @@ static GtkWidget *discard_create_dlg(gint num)
 {
 	GtkWidget *dlg_vbox;
 	GtkWidget *vbox;
-	char buff[128];
+	gchar *text;
 
-	discard.dlg = gtk_dialog_new_with_buttons(_("Discard resources"),
-						  GTK_WINDOW(app_window),
-						  GTK_DIALOG_DESTROY_WITH_PARENT,
-						  GTK_STOCK_OK,
-						  GTK_RESPONSE_OK, NULL);
+	discard.dlg = gtk_dialog_new_with_buttons(
+							 /* Dialog caption */
+							 _(""
+							   "Discard Resources"),
+							 GTK_WINDOW
+							 (app_window),
+							 GTK_DIALOG_DESTROY_WITH_PARENT,
+							 GTK_STOCK_OK,
+							 GTK_RESPONSE_OK,
+							 NULL);
 	g_signal_connect(G_OBJECT(discard.dlg), "destroy",
 			 G_CALLBACK(gtk_widget_destroyed), &discard.dlg);
 	gtk_widget_realize(discard.dlg);
@@ -93,11 +98,15 @@ static GtkWidget *discard_create_dlg(gint num)
 	gtk_box_pack_start(GTK_BOX(dlg_vbox), vbox, FALSE, TRUE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 6);
 
-	sprintf(buff, _("You must discard %d resources"), num);
+	text = g_strdup_printf(ngettext("You must discard %d resource",
+					"You must discard %d resources",
+					num), num);
 	discard.resource_widget =
-	    resource_table_new(buff, RESOURCE_TABLE_LESS_IN_HAND, FALSE,
+	    resource_table_new(text, RESOURCE_TABLE_LESS_IN_HAND, FALSE,
 			       TRUE);
+	g_free(text);
 	resource_table_set_total(RESOURCETABLE(discard.resource_widget),
+				 /* Label */
 				 _("Total discards"), num);
 	gtk_widget_show(discard.resource_widget);
 	gtk_box_pack_start(GTK_BOX(vbox), discard.resource_widget, FALSE,
@@ -251,7 +260,6 @@ GtkWidget *discard_build_page(void)
 							  NULL);
 	gtk_tree_view_column_set_sizing(column,
 					GTK_TREE_VIEW_COLUMN_GROW_ONLY);
-	set_pixbuf_tree_view_column_autogrow(discard_widget, column);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(discard_widget), column);
 
 	column = gtk_tree_view_column_new_with_attributes("",
