@@ -39,6 +39,8 @@
 #include "config-gnome.h"
 #include "gtkbugs.h"
 #include "audio.h"
+#include "notification.h"
+#include "gtkcompat.h"
 
 static GtkWidget *preferences_dlg;
 GtkWidget *app_window;		/* main application window */
@@ -101,21 +103,6 @@ static const gchar *pioneers_pixmaps[] = {
 	PIONEERS_PIXMAP_FINISH
 };
 
-static const gchar *resources_pixmaps[] = {
-	PIONEERS_PIXMAP_BRICK,
-	PIONEERS_PIXMAP_GRAIN,
-	PIONEERS_PIXMAP_ORE,
-	PIONEERS_PIXMAP_WOOL,
-	PIONEERS_PIXMAP_LUMBER
-};
-
-static struct {
-	GdkPixmap *p;
-	GdkBitmap *b;
-	GdkGC *gcp, *gcb;
-} resource_pixmap[NO_RESOURCE];
-static gint resource_pixmap_res = 0;
-
 static void gui_set_toolbar_visible(void);
 static void gui_toolbar_show_accelerators(gboolean show_accelerators);
 
@@ -210,68 +197,198 @@ static void game_settings_cb(void);
 static void help_manual_cb(void);
 #endif
 
+/** Toggles full screen mode.
+ * @param GtkToggleAction The calling action.
+ * @param main_window The window to toggle full screen mode.
+*/
+static void toggle_full_screen_cb(GtkToggleAction * caller,
+				  gpointer main_window)
+{
+	if (gtk_toggle_action_get_active(caller)) {
+		gtk_window_fullscreen(GTK_WINDOW(main_window));
+	} else {
+		gtk_window_unfullscreen(GTK_WINDOW(main_window));
+	}
+}
+
+static void zoom_normal_cb(void)
+{
+	guimap_zoom_normal(gmap);
+}
+
+static void zoom_center_map_cb(void)
+{
+	guimap_zoom_center_map(gmap);
+}
+
 /* Normal items */
 static GtkActionEntry entries[] = {
-	{"GameMenu", NULL, N_("_Game"), NULL, NULL, NULL},
-	{"GameNew", GTK_STOCK_NEW, N_("_New game"), "<control>N",
+	{"GameMenu", NULL,
+	 /* Menu entry */
+	 N_("_Game"), NULL, NULL, NULL},
+	{"GameNew", GTK_STOCK_NEW,
+	 /* Menu entry */
+	 N_("_New Game"), "<control>N",
+	 /* Tooltip for New Game menu entry */
 	 N_("Start a new game"), game_new_cb},
-	{"GameLeave", GTK_STOCK_STOP, N_("_Leave game"), NULL,
+	{"GameLeave", GTK_STOCK_STOP,
+	 /* Menu entry */
+	 N_("_Leave Game"), NULL,
+	 /* Tooltip for Leave Game menu entry */
 	 N_("Leave this game"), game_leave_cb},
 #ifdef ADMIN_GTK
-	{"GameAdmin", NULL, N_("_Admin"), "<control>A",
+	{"GameAdmin", NULL,
+	 /* Menu entry */
+	 N_("_Admin"), "<control>A",
+	 /* Tooltip for Admin menu entry */
 	 N_("Administer Pioneers server"), show_admin_interface},
 #endif
-	{"PlayerName", NULL, N_("_Player name"), "<control>P",
+	{"PlayerName", NULL,
+	 /* Menu entry */
+	 N_("_Player Name"), "<control>P",
+	 /* Tooltip for Player Name menu entry */
 	 N_("Change your player name"), playername_cb},
-	{"Legend", GTK_STOCK_DIALOG_INFO, N_("L_egend"), NULL,
+	{"Legend", GTK_STOCK_DIALOG_INFO,
+	 /* Menu entry */
+	 N_("L_egend"), NULL,
+	 /* Tooltip for Legend menu entry */
 	 N_("Terrain legend and building costs"), game_legend_cb},
-	{"GameSettings", GTK_STOCK_DIALOG_INFO, N_("_Game Settings"), NULL,
+	{"GameSettings", GTK_STOCK_DIALOG_INFO,
+	 /* Menu entry */
+	 N_("_Game Settings"), NULL,
+	 /* Tooltip for Game Settings menu entry */
 	 N_("Settings for the current game"), game_settings_cb},
-	{"DiceHistogram", GTK_STOCK_DIALOG_INFO, N_("_Dice Histogram"),
-	 NULL, N_("Histogram of dice rolls"), game_histogram_cb},
-	{"GameQuit", GTK_STOCK_QUIT, N_("_Quit"), "<control>Q",
+	{"DiceHistogram", GTK_STOCK_DIALOG_INFO,
+	 /* Menu entry */
+	 N_("_Dice Histogram"), NULL,
+	 /* Tooltip for Dice Histogram menu entry */
+	 N_("Histogram of dice rolls"), game_histogram_cb},
+	{"GameQuit", GTK_STOCK_QUIT,
+	 /* Menu entry */
+	 N_("_Quit"), "<control>Q",
+	 /* Tooltip for Quit menu entry */
 	 N_("Quit the program"), game_quit_cb},
-	{"ActionsMenu", NULL, N_("_Actions"), NULL, NULL, NULL},
-	{"RollDice", PIONEERS_PIXMAP_DICE, N_("Roll Dice"), "F1",
+	{"ActionsMenu", NULL,
+	 /* Menu entry */
+	 N_("_Actions"), NULL, NULL, NULL},
+	{"RollDice", PIONEERS_PIXMAP_DICE,
+	 /* Menu entry */
+	 N_("Roll Dice"), "F1",
+	 /* Tooltip for Roll Dice menu entry */
 	 N_("Roll the dice"), roll_dice_cb},
-	{"Trade", PIONEERS_PIXMAP_TRADE, N_("Trade"), "F2", N_("Trade"),
+	{"Trade", PIONEERS_PIXMAP_TRADE,
+	 /* Menu entry */
+	 N_("Trade"), "F2",
+	 /* Tooltip for Trade menu entry */
+	 N_("Trade"),
 	 trade_cb},
-	{"Undo", GTK_STOCK_UNDO, N_("Undo"), "F3", N_("Undo"), undo_cb},
-	{"Finish", PIONEERS_PIXMAP_FINISH, N_("Finish"), "F4",
+	{"Undo", GTK_STOCK_UNDO,
+	 /* Menu entry */
+	 N_("Undo"), "F3",
+	 /* Tooltip for Undo menu entry */
+	 N_("Undo"), undo_cb},
+	{"Finish", PIONEERS_PIXMAP_FINISH,
+	 /* Menu entry */
+	 N_("Finish"), "F4",
+	 /* Tooltip for Finish menu entry */
 	 N_("Finish"), finish_cb},
-	{"BuildRoad", PIONEERS_PIXMAP_ROAD, N_("Road"), "F5",
+	{"BuildRoad", PIONEERS_PIXMAP_ROAD,
+	 /* Menu entry */
+	 N_("Road"), "F5",
+	 /* Tooltip for Road menu entry */
 	 N_("Build a road"), build_road_cb},
-	{"BuildShip", PIONEERS_PIXMAP_SHIP, N_("Ship"), "F6",
+	{"BuildShip", PIONEERS_PIXMAP_SHIP,
+	 /* Menu entry */
+	 N_("Ship"), "F6",
+	 /* Tooltip for Ship menu entry */
 	 N_("Build a ship"), build_ship_cb},
-	{"MoveShip", PIONEERS_PIXMAP_SHIP_MOVEMENT, N_("Move Ship"), "F7",
+	{"MoveShip", PIONEERS_PIXMAP_SHIP_MOVEMENT,
+	 /* Menu entry */
+	 N_("Move Ship"), "F7",
+	 /* Tooltip for Move Ship menu entry */
 	 N_("Move a ship"), move_ship_cb},
-	{"BuildBridge", PIONEERS_PIXMAP_BRIDGE, N_("Bridge"), "F8",
+	{"BuildBridge", PIONEERS_PIXMAP_BRIDGE,
+	 /* Menu entry */
+	 N_("Bridge"), "F8",
+	 /* Tooltip for Bridge menu entry */
 	 N_("Build a bridge"), build_bridge_cb},
-	{"BuildSettlement", PIONEERS_PIXMAP_SETTLEMENT, N_("Settlement"),
-	 "F9", N_("Build a settlement"), build_settlement_cb},
-	{"BuildCity", PIONEERS_PIXMAP_CITY, N_("City"), "F10",
+	{"BuildSettlement", PIONEERS_PIXMAP_SETTLEMENT,
+	 /* Menu entry */
+	 N_("Settlement"),
+	 "F9",
+	 /* Tooltip for Settlement menu entry */
+	 N_("Build a settlement"), build_settlement_cb},
+	{"BuildCity", PIONEERS_PIXMAP_CITY,
+	 /* Menu entry */
+	 N_("City"), "F10",
+	 /* Tooltip for City menu entry */
 	 N_("Build a city"), build_city_cb},
-	{"BuyDevelopment", PIONEERS_PIXMAP_DEVELOP, N_("Develop"), "F11",
+	{"BuyDevelopment", PIONEERS_PIXMAP_DEVELOP,
+	 /* Menu entry */
+	 N_("Develop"), "F11",
+	 /* Tooltip for Develop menu entry */
 	 N_("Buy a development card"), buy_development_cb},
-	{"BuildCityWall", PIONEERS_PIXMAP_CITY_WALL, N_("City Wall"), NULL,
+	{"BuildCityWall", PIONEERS_PIXMAP_CITY_WALL,
+	 /* Menu entry */
+	 N_("City Wall"), NULL,
+	 /* Tooltip for City Wall menu entry */
 	 N_("Build a city wall"), build_city_wall_cb},
 
-	{"SettingsMenu", NULL, N_("_Settings"), NULL, NULL, NULL},
-	{"Preferences", GTK_STOCK_PREFERENCES, N_("Prefere_nces"), NULL,
+	{"SettingsMenu", NULL,
+	 /* Menu entry */
+	 N_("_Settings"), NULL, NULL, NULL},
+	{"Preferences", GTK_STOCK_PREFERENCES,
+	 /* Menu entry */
+	 N_("Prefere_nces"), NULL,
+	 /* Tooltip for Preferences menu entry */
 	 N_("Configure the application"), preferences_cb},
 
-	{"HelpMenu", NULL, N_("_Help"), NULL, NULL, NULL},
-	{"HelpAbout", NULL, N_("_About Pioneers"), NULL,
+	{"ViewMenu", NULL,
+	 /* Menu entry */
+	 N_("_View"), NULL, NULL, NULL},
+	{"Full", GTK_STOCK_ZOOM_FIT,
+	 /* Menu entry */
+	 N_("_Reset"),
+	 "<control>0",
+	 /* Tooltip for Reset menu entry */
+	 N_("View the full map"), zoom_normal_cb},
+	{"Center", NULL,
+	 /* Menu entry */
+	 N_("_Center"), NULL,
+	 /* Tooltip for Center menu entry */
+	 N_("Center the map"), zoom_center_map_cb},
+
+	{"HelpMenu", NULL,
+	 /* Menu entry */
+	 N_("_Help"), NULL, NULL, NULL},
+	{"HelpAbout", NULL,
+	 /* Menu entry */
+	 N_("_About Pioneers"), NULL,
+	 /* Tooltip for About Pioneers menu entry */
 	 N_("Information about Pioneers"), help_about_cb},
 #ifdef HAVE_HELP
-	{"HelpManual", GTK_STOCK_HELP, N_("_Help"), "<control>H",
+	{"HelpManual", GTK_STOCK_HELP,
+	 /* Menu entry */
+	 N_("_Help"), "<control>H",
+	 /* Tooltip for Help menu entry */
 	 N_("Show the manual"), help_manual_cb}
 #endif
 };
 
 /* Toggle items */
 static GtkToggleActionEntry toggle_entries[] = {
-	{"ShowHideToolbar", NULL, N_("_Toolbar"), NULL,
+	{"FullScreen", GTK_STOCK_FULLSCREEN,
+	 /* Menu entry */
+	 N_("_Fullscreen"),
+	 "<alt>Return",
+	 /* Tooltip for Fullscreen menu entry */
+	 N_("Set window to full screen mode"),
+	 G_CALLBACK(toggle_full_screen_cb),
+	 FALSE},
+	{"ShowHideToolbar", NULL,
+	 /* Menu entry */
+	 N_("_Toolbar"), NULL,
+	 /* Tooltip for Toolbar menu entry */
 	 N_("Show or hide the toolbar"), showhide_toolbar_cb, TRUE}
 };
 
@@ -313,6 +430,11 @@ static const char *ui_description =
 "      <menuitem action='ShowHideToolbar'/>"
 "      <menuitem action='Preferences'/>"
 "    </menu>"
+"    <menu action='ViewMenu'>"
+"      <menuitem action='FullScreen'/>"
+"      <menuitem action='Full'/>"
+"      <menuitem action='Center'/>"
+"    </menu>"
 "    <menu action='HelpMenu'>"
 "      <menuitem action='HelpAbout'/>"
 #ifdef HAVE_HELP
@@ -344,10 +466,13 @@ GtkWidget *gui_get_dialog_button(GtkDialog * dlg, gint button)
 	g_return_val_if_fail(dlg != NULL, NULL);
 	g_assert(dlg->action_area != NULL);
 
-	list = g_list_nth(GTK_BOX(dlg->action_area)->children, button);
+	list =
+	    gtk_container_get_children(GTK_CONTAINER
+				       (gtk_dialog_get_action_area(dlg)));
+	list = g_list_nth(list, button);
 	if (list != NULL) {
 		g_assert(list->data != NULL);
-		return ((GtkBoxChild *) list->data)->widget;
+		return GTK_WIDGET(list->data);
 	}
 	return NULL;
 }
@@ -367,7 +492,7 @@ void gui_set_vp_target_value(gint vp)
 	gchar *vp_text;
 
 	/* Victory points target in statusbar */
-	vp_text = g_strdup_printf(_("Points Needed to Win: %i"), vp);
+	vp_text = g_strdup_printf(_("Points needed to win: %i"), vp);
 
 	gtk_label_set_text(GTK_LABEL(vp_target_status), vp_text);
 	g_free(vp_text);
@@ -572,6 +697,8 @@ static GtkWidget *build_map_panel(void)
 {
 	GtkWidget *lbl;
 	GtkWidget *legend_content;
+	GtkWidget *hbox;
+	GtkWidget *close_button;
 
 	map_notebook = gtk_notebook_new();
 	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(map_notebook), GTK_POS_TOP);
@@ -583,20 +710,29 @@ static GtkWidget *build_map_panel(void)
 	gtk_notebook_insert_page(GTK_NOTEBOOK(map_notebook),
 				 build_map_area(), lbl, MAP_PAGE);
 
-	/* Tab page name */
-	lbl = gtk_label_new(_("Trade"));
-	gtk_widget_show(lbl);
+	hbox = create_label_with_close_button(
+						     /* Tab page name */
+						     _("Trade"),
+						     /* Tooltip */
+						     _("Finish trading"),
+						     &close_button);
+	frontend_gui_register(close_button, GUI_TRADE_FINISH, "clicked");
 	trade_page = trade_build_page();
 	gtk_notebook_insert_page(GTK_NOTEBOOK(map_notebook),
-				 trade_page, lbl, TRADE_PAGE);
+				 trade_page, hbox, TRADE_PAGE);
 	gtk_widget_hide(trade_page);
 
-	/* Tab page name */
-	lbl = gtk_label_new(_("Quote"));
-	gtk_widget_show(lbl);
+	hbox = create_label_with_close_button(
+						     /* Tab page name */
+						     _("Quote"),
+						     /* Tooltip */
+						     _(""
+						       "Reject domestic trade"),
+						     &close_button);
+	frontend_gui_register(close_button, GUI_QUOTE_REJECT, "clicked");
 	quote_page = quote_build_page();
 	gtk_notebook_insert_page(GTK_NOTEBOOK(map_notebook),
-				 quote_page, lbl, QUOTE_PAGE);
+				 quote_page, hbox, QUOTE_PAGE);
 	gtk_widget_hide(quote_page);
 
 	/* Tab page name */
@@ -795,8 +931,8 @@ static void quit_cb(G_GNUC_UNUSED GtkWidget * widget,
 
 static void theme_change_cb(GtkWidget * widget, G_GNUC_UNUSED void *data)
 {
-	gint index = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
-	MapTheme *theme = g_list_nth_data(theme_get_list(), index);
+	gint idx = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
+	MapTheme *theme = g_list_nth_data(theme_get_list(), idx);
 	if (theme != theme_get_current()) {
 		config_set_string("settings/theme", theme->name);
 		theme_set_current(theme);
@@ -869,6 +1005,14 @@ static void toggle_16_9_cb(GtkToggleButton * widget,
 	set_16_9_layout(layout_16_9);
 }
 
+static void toggle_notifications_cb(GtkToggleButton * widget,
+				    G_GNUC_UNUSED gpointer user_data)
+{
+	gboolean show_notifications = gtk_toggle_button_get_active(widget);
+	config_set_int("settings/show_notifications", show_notifications);
+	set_show_notifications(show_notifications);
+}
+
 static void showhide_toolbar_cb(void)
 {
 	gui_set_toolbar_visible();
@@ -926,7 +1070,7 @@ static void preferences_cb(void)
 
 	row = 0;
 
-	theme_list = gtk_combo_box_new_text();
+	theme_list = gtk_combo_box_text_new();
 	/* Label for changing the theme, in the preferences dialog */
 	theme_label = gtk_label_new(_("Theme:"));
 	gtk_misc_set_alignment(GTK_MISC(theme_label), 0, 0.5);
@@ -936,8 +1080,8 @@ static void preferences_cb(void)
 	for (i = 0, theme_elt = theme_get_list();
 	     theme_elt != NULL; ++i, theme_elt = g_list_next(theme_elt)) {
 		MapTheme *theme = theme_elt->data;
-		gtk_combo_box_append_text(GTK_COMBO_BOX(theme_list),
-					  theme->name);
+		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT
+					       (theme_list), theme->name);
 		if (theme == theme_get_current())
 			gtk_combo_box_set_active(GTK_COMBO_BOX(theme_list),
 						 i);
@@ -1044,7 +1188,7 @@ static void preferences_cb(void)
 	row++;
 
 	widget =
-	    /* Label for the option to announce when players/viewers enter */
+	    /* Label for the option to announce when players/spectators enter */
 	    gtk_check_button_new_with_label(_("Announce new players"));
 	gtk_widget_show(widget);
 	gtk_table_attach_defaults(GTK_TABLE(layout), widget,
@@ -1054,9 +1198,9 @@ static void preferences_cb(void)
 	g_signal_connect(G_OBJECT(widget), "toggled",
 			 G_CALLBACK(announce_player_cb), NULL);
 	gtk_widget_set_tooltip_text(widget,
-				    /* Tooltip for the option to use sound when players/viewers enter */
+				    /* Tooltip for the option to use sound when players/spectators enter */
 				    _(""
-				      "Make a sound when a new player or viewer enters the game"));
+				      "Make a sound when a new player or spectator enters the game"));
 	row++;
 
 	/* Silent mode widget is connected an initialized after the announce button */
@@ -1065,6 +1209,23 @@ static void preferences_cb(void)
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(silent_mode_widget),
 				     get_silent_mode());
 
+#ifdef HAVE_NOTIFY
+	/* Label for the option to use the notifications. */
+	widget = gtk_check_button_new_with_label(_("Show notifications"));
+	gtk_widget_show(widget);
+	gtk_table_attach_defaults(GTK_TABLE(layout), widget,
+				  0, 2, row, row + 1);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget),
+				     get_show_notifications());
+	g_signal_connect(G_OBJECT(widget), "toggled",
+			 G_CALLBACK(toggle_notifications_cb), NULL);
+	gtk_widget_set_tooltip_text(widget,
+				    /* Tooltip for notifications option. */
+				    _("Show notifications when it's your "
+				      "turn or when new trade is available"));
+	row++;
+#endif
+
 	/* Label for the option to use the 16:9 layout. */
 	widget = gtk_check_button_new_with_label(_("Use 16:9 layout"));
 	gtk_widget_show(widget);
@@ -1079,7 +1240,6 @@ static void preferences_cb(void)
 				    _(""
 				      "Use a 16:9 friendly layout for the window"));
 	row++;
-
 }
 
 static void help_about_cb(void)
@@ -1087,7 +1247,9 @@ static void help_about_cb(void)
 	const gchar *authors[] = {
 		AUTHORLIST
 	};
-	aboutbox_display(_("The Pioneers Game"), authors);
+	aboutbox_display(
+				/* Caption of about box */
+				_("About Pioneers"), authors);
 }
 
 static void game_legend_cb(void)
@@ -1413,12 +1575,11 @@ static void register_pixmaps(void)
 		gchar *filename;
 		GtkIconSet *icon;
 
-		icon = gtk_icon_set_new();
+		icon = NULL;
 		/* determine full path to pixmap file */
 		filename = g_build_filename(DATADIR, "pixmaps",
 					    pioneers_pixmaps[idx], NULL);
 		if (g_file_test(filename, G_FILE_TEST_EXISTS)) {
-			GtkIconSource *source;
 			GdkPixbuf *pixbuf;
 			GError *error = NULL;
 			pixbuf =
@@ -1428,11 +1589,8 @@ static void register_pixmaps(void)
 					  filename);
 				g_error_free(error);
 			} else {
-				source = gtk_icon_source_new();
-				gtk_icon_source_set_pixbuf(source, pixbuf);
-				g_object_unref(pixbuf);
-				gtk_icon_set_add_source(icon, source);
-				gtk_icon_source_free(source);
+				icon =
+				    gtk_icon_set_new_from_pixbuf(pixbuf);
 			}
 		} else {
 			/* Missing pixmap */
@@ -1446,51 +1604,6 @@ static void register_pixmaps(void)
 
 	gtk_icon_factory_add_default(factory);
 	g_object_unref(factory);
-
-	for (idx = 0; idx < NO_RESOURCE; idx++) {
-		gchar *filename;
-
-		/* determine full path to pixmap file */
-		filename = g_build_filename(DATADIR, "pixmaps",
-					    resources_pixmaps[idx], NULL);
-		if (g_file_test(filename, G_FILE_TEST_EXISTS)) {
-			GdkPixbuf *pixbuf;
-			GError *error = NULL;
-			pixbuf =
-			    gdk_pixbuf_new_from_file(filename, &error);
-			if (error != NULL) {
-				g_warning("Error loading pixmap %s\n",
-					  filename);
-				g_error_free(error);
-			} else {
-				gdk_pixbuf_render_pixmap_and_mask(pixbuf,
-								  &resource_pixmap
-								  [idx].p,
-								  &resource_pixmap
-								  [idx].b,
-								  128);
-				resource_pixmap[idx].gcb =
-				    gdk_gc_new(resource_pixmap[idx].b);
-				gdk_gc_set_function(resource_pixmap
-						    [idx].gcb, GDK_OR);
-				resource_pixmap[idx].gcp =
-				    gdk_gc_new(resource_pixmap[idx].p);
-				gdk_gc_set_clip_mask(resource_pixmap
-						     [idx].gcp,
-						     resource_pixmap
-						     [idx].b);
-				if (!resource_pixmap_res)
-					resource_pixmap_res =
-					    gdk_pixbuf_get_width(pixbuf);
-				g_object_unref(pixbuf);
-			}
-		} else {
-			/* Missing pixmap */
-			g_warning("Pixmap not found: %s", filename);
-		}
-
-		g_free(filename);
-	}
 }
 
 GtkWidget *gui_build_interface(void)
@@ -1511,6 +1624,8 @@ GtkWidget *gui_build_interface(void)
 	/* The name of the application */
 	gtk_window_set_title(GTK_WINDOW(app_window), _("Pioneers"));
 
+	prepare_gtk_for_close_button_on_tab();
+
 	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_widget_show(vbox);
 	gtk_container_add(GTK_CONTAINER(app_window), vbox);
@@ -1561,6 +1676,8 @@ GtkWidget *gui_build_interface(void)
 			("settings/silent_mode", FALSE));
 	set_announce_player(config_get_int_with_default
 			    ("settings/announce_player", TRUE));
+	set_show_notifications(config_get_int_with_default
+			       ("settings/show_notifications", TRUE));
 
 	legend_page_enabled =
 	    config_get_int_with_default("settings/legend_page", FALSE);
@@ -1635,23 +1752,6 @@ GtkWidget *gui_build_interface(void)
 	return app_window;
 }
 
-void gui_get_resource_pixmap(gint idx, GdkPixmap ** p, GdkBitmap ** b,
-			     GdkGC ** gcp, GdkGC ** gcb)
-{
-	g_assert(idx < NO_RESOURCE);
-	*p = resource_pixmap[idx].p;
-	*b = resource_pixmap[idx].b;
-	if (gcp)
-		*gcp = resource_pixmap[idx].gcp;
-	if (gcb)
-		*gcb = resource_pixmap[idx].gcb;
-}
-
-gint gui_get_resource_pixmap_res()
-{
-	return resource_pixmap_res;
-}
-
 void gui_set_show_no_setup_nodes(gboolean show)
 {
 	guimap_set_show_no_setup_nodes(gmap, show);
