@@ -24,6 +24,7 @@
 #include "frontend.h"
 #include "theme.h"
 #include "cost.h"
+#include "resource-view.h"
 
 /* The order of the terrain_names is EXTREMELY important!  The order
  * must match the enum Terrain.
@@ -45,45 +46,11 @@ static gboolean legend_did_connect = FALSE;
 static void legend_theme_changed(void);
 static void legend_rules_changed(void);
 
-static gint expose_legend_cb(GtkWidget * area,
-			     G_GNUC_UNUSED GdkEventExpose * event,
-			     gpointer terraindata)
-{
-	MapTheme *theme = theme_get_current();
-	static GdkGC *legend_gc;
-	GdkPixbuf *p;
-	gint height;
-	Terrain terrain = GPOINTER_TO_INT(terraindata);
-
-	if (area->window == NULL)
-		return FALSE;
-
-	if (legend_gc == NULL)
-		legend_gc = gdk_gc_new(area->window);
-
-	gdk_gc_set_fill(legend_gc, GDK_TILED);
-	gdk_gc_set_tile(legend_gc, guimap_terrain(terrain));
-
-	height = area->allocation.width / theme->scaledata[terrain].aspect;
-	p = gdk_pixbuf_scale_simple(theme->scaledata[terrain].native_image,
-				    area->allocation.width, height,
-				    GDK_INTERP_BILINEAR);
-
-	/* Center the image in the available space */
-	gdk_draw_pixbuf(area->window, legend_gc, p,
-			0, 0, 0, (area->allocation.height - height) / 2,
-			-1, -1, GDK_RGB_DITHER_NONE, 0, 0);
-	gdk_pixbuf_unref(p);
-	return FALSE;
-}
-
 static void add_legend_terrain(GtkWidget * table, guint row, guint col,
 			       Terrain terrain, Resource resource)
 {
 	GtkWidget *area;
 	GtkWidget *label;
-	GdkPixmap *p;
-	GdkBitmap *b;
 
 	area = gtk_drawing_area_new();
 	gtk_widget_show(area);
@@ -93,7 +60,7 @@ static void add_legend_terrain(GtkWidget * table, guint row, guint col,
 			 (GtkAttachOptions) GTK_FILL, 0, 0);
 	gtk_widget_set_size_request(area, 30, 34);
 	g_signal_connect(G_OBJECT(area), "expose_event",
-			 G_CALLBACK(expose_legend_cb),
+			 G_CALLBACK(expose_terrain_cb),
 			 GINT_TO_POINTER(terrain));
 
 	label = gtk_label_new(_(terrain_names[terrain]));
@@ -105,16 +72,14 @@ static void add_legend_terrain(GtkWidget * table, guint row, guint col,
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 
 	if (resource < NO_RESOURCE) {
-		gui_get_resource_pixmap(resource, &p, &b, NULL, NULL);
-		label = gtk_image_new_from_pixmap(p, b);
+		label = resource_view_new_single_resource(resource);
 		gtk_widget_show(label);
 		gtk_table_attach(GTK_TABLE(table), label,
 				 col + 2, col + 3, row, row + 1,
 				 (GtkAttachOptions) GTK_FILL,
 				 (GtkAttachOptions) GTK_FILL, 0, 0);
-	}
+		gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 
-	if (resource != NO_RESOURCE) {
 		label = gtk_label_new(resource_name(resource, TRUE));
 		gtk_widget_show(label);
 		gtk_table_attach(GTK_TABLE(table), label,
@@ -142,8 +107,8 @@ static void add_legend_cost(GtkWidget * table, guint row,
 			 GTK_FILL, GTK_FILL, 0, 0);
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 
-	label = gtk_image_new();
-	resource_format_type_image(GTK_IMAGE(label), cost, -1);
+	label = resource_view_new();
+	resource_view_set(RESOURCE_VIEW(label), cost);
 	gtk_widget_show(label);
 	gtk_table_attach(GTK_TABLE(table), label, 2, 3, row, row + 1,
 			 GTK_FILL, GTK_FILL, 0, 0);
@@ -166,7 +131,8 @@ GtkWidget *legend_create_content(void)
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 6);
 
 	label = gtk_label_new(NULL);
-	gtk_label_set_markup(GTK_LABEL(label), _("<b>Terrain Yield</b>"));
+	/* Label */
+	gtk_label_set_markup(GTK_LABEL(label), _("<b>Terrain yield</b>"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 	gtk_widget_show(label);
 	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, TRUE, 0);
@@ -198,7 +164,8 @@ GtkWidget *legend_create_content(void)
 	add_legend_terrain(table, 3, 5, SEA_TERRAIN, NO_RESOURCE);
 
 	label = gtk_label_new(NULL);
-	gtk_label_set_markup(GTK_LABEL(label), _("<b>Building Costs</b>"));
+	/* Label */
+	gtk_label_set_markup(GTK_LABEL(label), _("<b>Building costs</b>"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 	gtk_widget_show(label);
 	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, TRUE, 0);
@@ -236,10 +203,10 @@ GtkWidget *legend_create_content(void)
 			cost_upgrade_settlement());
 	if (have_city_walls())
 		add_legend_cost(table, num_rows++,
-				PIONEERS_PIXMAP_CITY_WALL, _("City Wall"),
+				PIONEERS_PIXMAP_CITY_WALL, _("City wall"),
 				cost_city_wall());
 	add_legend_cost(table, num_rows++, PIONEERS_PIXMAP_DEVELOP,
-			_("Development Card"), cost_development());
+			_("Development card"), cost_development());
 
 	gtk_widget_show(vbox);
 	gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
@@ -257,6 +224,7 @@ GtkWidget *legend_create_dlg(void)
 		return legend_dlg;
 	}
 
+	/* Dialog caption */
 	legend_dlg = gtk_dialog_new_with_buttons(_("Legend"),
 						 GTK_WINDOW(app_window),
 						 GTK_DIALOG_DESTROY_WITH_PARENT,
