@@ -3,6 +3,7 @@
  *
  * Copyright (C) 1999 Dave Cole
  * Copyright (C) 2003,2006 Bas Wijnen <shevek@fmf.nl>
+ * Copyright (C) 2010 Roland Clobus <rclobus@rclobus.nl>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +25,7 @@
 #include "frontend.h"
 #include "player-icon.h"
 #include "config-gnome.h"
+#include "client.h"
 
 typedef struct {
 	GtkWidget *dlg;
@@ -37,27 +39,35 @@ typedef struct {
 	gchar *original_name;
 	gchar *original_style;
 	gchar *current_style;
+	gulong name_change_cb_id;
 } DialogData;
 
 static DialogData name_dialog;
 
 static GdkPixbuf *create_icon(GtkWidget * widget, const gchar * style);
 
+static void name_change_name_cb(NotifyingString * name)
+{
+	gchar *nm = notifying_string_get(name);
+	gtk_entry_set_text(GTK_ENTRY(name_dialog.name_entry), nm);
+	g_free(nm);
+}
+
 static void change_name_cb(G_GNUC_UNUSED GtkDialog * dlg, int response_id,
 			   gpointer user_data)
 {
 	DialogData *dialog = user_data;
+	g_signal_handler_disconnect(requested_name,
+				    dialog->name_change_cb_id);
 	if (response_id == GTK_RESPONSE_OK) {
 		const gchar *new_name =
 		    gtk_entry_get_text(GTK_ENTRY(dialog->name_entry));
 		const gchar *new_style = dialog->current_style;
 		if (0 != strcmp(new_name, dialog->original_name)) {
-			cb_name_change(new_name);
-			config_set_string("connect/name", new_name);
+			notifying_string_set(requested_name, new_name);
 		}
 		if (0 != strcmp(new_style, dialog->original_style)) {
-			cb_style_change(new_style);
-			config_set_string("connect/style", new_style);
+			notifying_string_set(requested_style, new_style);
 		}
 	}
 	g_free(dialog->original_name);
@@ -82,13 +92,13 @@ static void change_style_cb(G_GNUC_UNUSED GtkWidget * widget,
 					   (dialog->color_btn1), &c1);
 		gtk_color_button_get_color(GTK_COLOR_BUTTON
 					   (dialog->color_btn2), &c2);
-		variant = (int)
+		variant =
 		    gtk_range_get_value(GTK_RANGE(dialog->variant_btn)) -
 		    1;
 		dialog->current_style =
 		    playericon_create_human_style(&c1, variant, &c2);
 	} else {
-		dialog->current_style = g_strdup("square");
+		dialog->current_style = g_strdup(default_player_style);
 	}
 	icon = create_icon(dialog->image, dialog->current_style);
 	gtk_image_set_from_pixbuf(GTK_IMAGE(dialog->image), icon);
@@ -119,13 +129,18 @@ void name_create_dlg(void)
 		return;
 	};
 
-	name_dialog.dlg =
-	    gtk_dialog_new_with_buttons(_("Change player name"),
-					GTK_WINDOW(app_window),
-					GTK_DIALOG_DESTROY_WITH_PARENT,
-					GTK_STOCK_CANCEL,
-					GTK_RESPONSE_CANCEL, GTK_STOCK_OK,
-					GTK_RESPONSE_OK, NULL);
+	name_dialog.dlg = gtk_dialog_new_with_buttons(
+							     /* Dialog caption */
+							     _(""
+							       "Change Player Name"),
+							     GTK_WINDOW
+							     (app_window),
+							     GTK_DIALOG_DESTROY_WITH_PARENT,
+							     GTK_STOCK_CANCEL,
+							     GTK_RESPONSE_CANCEL,
+							     GTK_STOCK_OK,
+							     GTK_RESPONSE_OK,
+							     NULL);
 	gtk_dialog_set_default_response(GTK_DIALOG(name_dialog.dlg),
 					GTK_RESPONSE_OK);
 	g_signal_connect(G_OBJECT(name_dialog.dlg), "destroy",
@@ -143,7 +158,8 @@ void name_create_dlg(void)
 	gtk_box_pack_start(GTK_BOX(dlg_vbox), hbox, FALSE, TRUE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
 
-	lbl = gtk_label_new(_("Player Name:"));
+	/* Label */
+	lbl = gtk_label_new(_("Player name:"));
 	gtk_widget_show(lbl);
 	gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, TRUE, 0);
 	gtk_misc_set_alignment(GTK_MISC(lbl), 1, 0.5);
@@ -156,15 +172,18 @@ void name_create_dlg(void)
 			   TRUE, 0);
 	gtk_entry_set_width_chars(GTK_ENTRY(name_dialog.name_entry),
 				  MAX_NAME_LENGTH);
-	name_dialog.original_name = g_strdup(my_player_name());
+	name_dialog.original_name = notifying_string_get(requested_name);
 	gtk_entry_set_text(GTK_ENTRY(name_dialog.name_entry),
 			   name_dialog.original_name);
 
 	gtk_entry_set_activates_default(GTK_ENTRY(name_dialog.name_entry),
 					TRUE);
+	name_dialog.name_change_cb_id =
+	    g_signal_connect(requested_name, "changed",
+			     G_CALLBACK(name_change_name_cb), NULL);
 
-	name_dialog.original_style = g_strdup(my_player_style());
-	name_dialog.current_style = g_strdup(my_player_style());
+	name_dialog.original_style = notifying_string_get(requested_style);
+	name_dialog.current_style = notifying_string_get(requested_style);
 	parse_ok =
 	    playericon_parse_human_style(name_dialog.current_style,
 					 &face_color, &variant,
@@ -198,7 +217,7 @@ void name_create_dlg(void)
 	gtk_container_set_border_width(GTK_CONTAINER
 				       (name_dialog.style_hbox), 5);
 
-	/* Set icon preferences */
+	/* Label: set player icon preferences */
 	lbl = gtk_label_new(_("Face:"));
 	gtk_widget_show(lbl);
 	gtk_box_pack_start(GTK_BOX(name_dialog.style_hbox), lbl, FALSE,
@@ -214,6 +233,7 @@ void name_create_dlg(void)
 	g_signal_connect(name_dialog.color_btn1, "color-set",
 			 G_CALLBACK(change_style_cb), &name_dialog);
 
+	/* Label: set player icon preferences */
 	lbl = gtk_label_new(_("Variant:"));
 	gtk_widget_show(lbl);
 	gtk_box_pack_start(GTK_BOX(name_dialog.style_hbox), lbl, FALSE,
@@ -255,7 +275,7 @@ void name_create_dlg(void)
 GdkPixbuf *create_icon(GtkWidget * widget, const gchar * style)
 {
 	return playericon_create_icon(widget, style,
-				      player_or_viewer_color(my_player_num
-							     ()),
-				      my_player_viewer(), TRUE, TRUE);
+				      player_or_spectator_color
+				      (my_player_num()),
+				      my_player_spectator(), TRUE, TRUE);
 }
