@@ -52,13 +52,13 @@ typedef struct {
 static Statistic statistics[] = {
 	{N_("Settlement"), N_("Settlements"), &ps_settlement},
 	{N_("City"), N_("Cities"), &ps_city},
-	{N_("City Wall"), N_("City Walls"), &ps_city_wall},
-	{N_("Largest Army"), NULL, &ps_largest},
-	{N_("Longest Road"), NULL, &ps_largest},
+	{N_("City wall"), N_("City walls"), &ps_city_wall},
+	{N_("Largest army"), NULL, &ps_largest},
+	{N_("Longest road"), NULL, &ps_largest},
 	{N_("Chapel"), N_("Chapels"), &ps_building},
-	{N_("Pioneer University"), N_("Pioneer Universities"),
+	{N_("Pioneer university"), N_("Pioneer universities"),
 	 &ps_building},
-	{N_("Governor's House"), N_("Governor's Houses"), &ps_building},
+	{N_("Governor's house"), N_("Governor's houses"), &ps_building},
 	{N_("Library"), N_("Libraries"), &ps_building},
 	{N_("Market"), N_("Markets"), &ps_building},
 	{N_("Soldier"), N_("Soldiers"), &ps_soldier},
@@ -99,7 +99,10 @@ struct Player_point {
 };
 
 static GtkWidget *turn_area;	/** turn indicator in status bar */
-static const gint turn_area_icon_width = 30;
+/** Width for each icon */
+static const gint turn_area_icon_width = 28;
+/** Separation between each icon */
+static const gint turn_area_icon_separation = 2;
 
 void player_init(void)
 {
@@ -112,10 +115,10 @@ GdkColor *player_color(gint player_num)
 	return colors_get_player(player_num);
 }
 
-GdkColor *player_or_viewer_color(gint player_num)
+GdkColor *player_or_spectator_color(gint player_num)
 {
-	if (player_is_viewer(player_num)) {
-		/* viewer color is always black */
+	if (player_is_spectator(player_num)) {
+		/* spectator color is always black */
 		return &black;
 	}
 	return colors_get_player(player_num);
@@ -125,8 +128,9 @@ GdkPixbuf *player_create_icon(GtkWidget * widget, gint player_num,
 			      gboolean connected)
 {
 	return playericon_create_icon(widget, player_get_style(player_num),
-				      player_or_viewer_color(player_num),
-				      player_is_viewer(player_num),
+				      player_or_spectator_color
+				      (player_num),
+				      player_is_spectator(player_num),
 				      connected, FALSE);
 }
 
@@ -386,7 +390,7 @@ static void player_create_find_player(gint player_num, GtkTreeIter * iter)
 	GtkTreeIter found_iter;
 	enum TFindResult result;
 
-	/* Search for a place to add information about the player/viewer */
+	/* Search for a place to add information about the player/spectator */
 	result =
 	    find_integer_in_tree(GTK_TREE_MODEL(summary_store),
 				 &found_iter, SUMMARY_COLUMN_PLAYER_NUM,
@@ -439,17 +443,17 @@ void frontend_player_style(gint player_num,
 	chat_player_style(player_num);
 }
 
-void frontend_viewer_name(gint viewer_num, const gchar * name)
+void frontend_spectator_name(gint spectator_num, const gchar * name)
 {
 	GtkTreeIter iter;
 
-	player_create_find_player(viewer_num, &iter);
+	player_create_find_player(spectator_num, &iter);
 	gtk_list_store_set(summary_store, &iter,
 			   SUMMARY_COLUMN_TEXT, name, -1);
 	if (callback_mode != MODE_INIT)
 		play_sound(SOUND_ANNOUNCE);
 
-	chat_player_name(viewer_num, name);
+	chat_player_name(spectator_num, name);
 }
 
 void frontend_player_quit(gint player_num)
@@ -462,14 +466,14 @@ void frontend_player_quit(gint player_num)
 	chat_player_quit(player_num);
 }
 
-void frontend_viewer_quit(gint viewer_num)
+void frontend_spectator_quit(gint spectator_num)
 {
 	GtkTreeIter iter;
 
-	player_create_find_player(viewer_num, &iter);
+	player_create_find_player(spectator_num, &iter);
 	gtk_list_store_remove(summary_store, &iter);
 
-	chat_viewer_quit(viewer_num);
+	chat_spectator_quit(spectator_num);
 }
 
 static void player_show_connected_at_iter(gint player_num,
@@ -548,7 +552,7 @@ GtkWidget *player_build_summary(void)
 
 	label = gtk_label_new(NULL);
 	/* Caption for the overview of the points and card of other players */
-	gtk_label_set_markup(GTK_LABEL(label), _("<b>Player Summary</b>"));
+	gtk_label_set_markup(GTK_LABEL(label), _("<b>Player summary</b>"));
 	gtk_widget_show(label);
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	gtk_container_add(GTK_CONTAINER(alignment), label);
@@ -570,7 +574,6 @@ GtkWidget *player_build_summary(void)
 							  NULL);
 	gtk_tree_view_column_set_sizing(column,
 					GTK_TREE_VIEW_COLUMN_GROW_ONLY);
-	set_pixbuf_tree_view_column_autogrow(summary_widget, column);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(summary_widget), column);
 
 	column = gtk_tree_view_column_new_with_attributes("",
@@ -620,46 +623,39 @@ static gint expose_turn_area_cb(GtkWidget * area,
 				G_GNUC_UNUSED GdkEventExpose * event,
 				G_GNUC_UNUSED gpointer user_data)
 {
-	static GdkGC *turn_gc;
+	cairo_t *cr;
 	gint offset;
 	gint idx;
 
 	if (area->window == NULL)
 		return FALSE;
 
-	if (turn_gc == NULL)
-		turn_gc = gdk_gc_new(area->window);
+	cr = gdk_cairo_create(area->window);
 
 	offset = 0;
 	for (idx = 0; idx < num_players(); idx++) {
-		gdk_gc_set_foreground(turn_gc, player_color(idx));
-		gdk_draw_rectangle(area->window, turn_gc, TRUE,
-				   offset + 2, 2,
-				   turn_area_icon_width - 4,
-				   area->allocation.height - 4);
-		gdk_gc_set_foreground(turn_gc, &black);
-		if (idx == current_player()) {
-			gdk_gc_set_line_attributes(turn_gc, 3,
-						   GDK_LINE_SOLID,
-						   GDK_CAP_BUTT,
-						   GDK_JOIN_MITER);
-			gdk_draw_rectangle(area->window, turn_gc, FALSE,
-					   offset + 3, 3,
-					   turn_area_icon_width - 6,
-					   area->allocation.height - 6);
-		} else {
-			gdk_gc_set_line_attributes(turn_gc, 1,
-						   GDK_LINE_SOLID,
-						   GDK_CAP_BUTT,
-						   GDK_JOIN_MITER);
-			gdk_draw_rectangle(area->window, turn_gc, FALSE,
-					   offset + 2, 2,
-					   turn_area_icon_width - 4,
-					   area->allocation.height - 4);
-		}
+		gdk_cairo_set_source_color(cr, player_color(idx));
+		cairo_rectangle(cr, offset, 0, turn_area_icon_width,
+				area->allocation.height);
+		cairo_fill(cr);
 
-		offset += turn_area_icon_width;
+		gdk_cairo_set_source_color(cr, &black);
+		if (idx == current_player()) {
+			cairo_set_line_width(cr, 3.0);
+			cairo_rectangle(cr, offset + 1.5, 1.5,
+					turn_area_icon_width - 3,
+					area->allocation.height - 3);
+		} else {
+			cairo_set_line_width(cr, 1.0);
+			cairo_rectangle(cr, offset + 0.5, 0.5,
+					turn_area_icon_width - 1,
+					area->allocation.height - 1);
+		}
+		cairo_stroke(cr);
+
+		offset += turn_area_icon_width + turn_area_icon_separation;
 	}
+	cairo_destroy(cr);
 
 	return FALSE;
 }
@@ -670,8 +666,9 @@ GtkWidget *player_build_turn_area(void)
 	g_signal_connect(G_OBJECT(turn_area), "expose_event",
 			 G_CALLBACK(expose_turn_area_cb), NULL);
 	gtk_widget_set_size_request(turn_area,
-				    turn_area_icon_width * num_players(),
-				    -1);
+				    turn_area_icon_width * num_players() +
+				    turn_area_icon_separation *
+				    (num_players() - 1), -1);
 	gtk_widget_show(turn_area);
 
 	return turn_area;
@@ -679,7 +676,9 @@ GtkWidget *player_build_turn_area(void)
 
 void set_num_players(gint num)
 {
-	gtk_widget_set_size_request(turn_area, turn_area_icon_width * num,
+	gtk_widget_set_size_request(turn_area,
+				    turn_area_icon_width * num +
+				    turn_area_icon_separation * (num - 1),
 				    -1);
 }
 
