@@ -87,8 +87,18 @@ typedef enum {
 	PARAMS_NO_WIN		/* the game cannot be won */
 } WinnableState;
 
+typedef enum {
+	PLAYER_HUMAN,		/* the player is a human */
+	PLAYER_COMPUTER,	/* the player is a computer player */
+	PLAYER_UNKNOWN		/* it is unknown who is controlling the player */
+} PlayerType;
+#define NUM_PLAYER_TYPES (PLAYER_UNKNOWN + 1)
+
 typedef void (*WriteLineFunc) (gpointer user_data, const gchar *);
 
+/** Default style for a player. */
+const gchar *default_player_style;
+
 GameParams *params_new(void);
 GameParams *params_copy(const GameParams * params);
 GameParams *params_load_file(const gchar * fname);
@@ -109,6 +119,67 @@ WinnableState params_check_winnable_state(const GameParams * params,
 					  gchar ** win_message,
 					  gchar ** point_specification);
 
+/** Determine the type of the player, by analysing the style. */
+PlayerType determine_player_type(const gchar * style);
+
 Points *points_new(gint id, const gchar * name, gint points);
 void points_free(Points * points);
+
+/* Communication format
+ *
+ * The commands sent to and from the server use the following
+ * format specifiers:
+ *	%S - string from current position to end of line
+ *		this takes a gchar ** argument, in which an allocated buffer
+ *		is returned.  It must be freed by the caller.
+ *	%d - integer
+ *	%B - build type:
+ *		'road' = BUILD_ROAD
+ *		'ship' = BUILD_SHIP
+ *		'bridge' = BUILD_BRIDGE
+ *		'settlement' = BUILD_SETTLEMENT
+ *		'city' = BUILD_CITY
+ *	%R - list of 5 integer resource counts:
+ *		brick, grain, ore, wool, lumber
+ *	%D - development card type:
+ *		0 = DEVEL_ROAD_BUILDING
+ *		1 = DEVEL_MONOPOLY
+ *		2 = DEVEL_YEAR_OF_PLENTY
+ *		3 = DEVEL_CHAPEL
+ *		4 = DEVEL_UNIVERSITY
+ *		5 = DEVEL_GOVERNORS_HOUSE
+ *		6 = DEVEL_LIBRARY
+ *		7 = DEVEL_MARKET
+ *		8 = DEVEL_SOLDIER
+ *	%r - resource type:
+ *		'brick' = BRICK_RESOURCE
+ *		'grain' = GRAIN_RESOURCE
+ *		'ore' = ORE_RESOURCE
+ *		'wool' = WOOL_RESOURCE
+ *		'lumber' = LUMBER_RESOURCE
+ */
+/** Parse a line.
+ * @param line Line to parse
+ * @param fmt Format of the line, see communication format
+ * @retval ap Result of the parse
+ * @return -1 if the line could not be parsed, otherwise the offset in the line
+*/
+gint game_vscanf(const gchar * line, const gchar * fmt, va_list ap);
+/** Parse a line.
+ * @param line Line to parse
+ * @param fmt Format of the line, see communication format
+ * @return -1 if the line could not be parsed, otherwise the offset in the line
+*/
+gint game_scanf(const gchar * line, const gchar * fmt, ...);
+/** Print a line.
+ * @param fmt Format of the line, see communication format
+ * @param ap Arguments to the format
+ * @return A string (you must use g_free to free the string)
+*/
+gchar *game_vprintf(const gchar * fmt, va_list ap);
+/** Print a line.
+ * @param fmt Format of the line, see communication format
+ * @return A string (you must use g_free to free the string)
+*/
+gchar *game_printf(const gchar * fmt, ...);
 #endif
