@@ -105,7 +105,7 @@ void message_window_log_message_string(gint msg_type, const gchar * text)
 			case MSG_CHAT:
 				tagname = "chat";
 				break;
-			case MSG_VIEWER_CHAT:
+			case MSG_SPECTATOR_CHAT:
 				tagname = "chat";
 				break;
 			case MSG_RESOURCE:
@@ -315,7 +315,10 @@ void check_victory_points(GameParams * params, GtkWindow * main_window)
 	dialog = gtk_message_dialog_new(main_window,
 					GTK_DIALOG_DESTROY_WITH_PARENT,
 					message_type,
-					GTK_BUTTONS_OK, win_message);
+					GTK_BUTTONS_OK, "%s", win_message);
+	gtk_window_set_title(GTK_WINDOW(dialog),
+			     /* Caption for result of checking victory points */
+			     _("Victory Point Analysis"));
 
 	gtk_message_dialog_format_secondary_text
 	    (GTK_MESSAGE_DIALOG(dialog), "%s", point_specification);
@@ -325,3 +328,79 @@ void check_victory_points(GameParams * params, GtkWindow * main_window)
 	g_free(win_message);
 	g_free(point_specification);
 }
+
+void prepare_gtk_for_close_button_on_tab(void)
+{
+	gtk_rc_parse_string("style \"tab-with-close-button-style\"\n"
+			    "{\n"
+			    "  GtkWidget::focus-padding = 0\n"
+			    "  GtkWidget::focus-line-width = 0\n"
+			    "  xthickness = 0\n"
+			    "  ythickness = 0\n"
+			    "}\n"
+			    "widget \"*.tab-with-close-button\" style \"tab-with-close-button-style\"");
+}
+
+GtkWidget *create_label_with_close_button(const gchar * label_text,
+					  const gchar * tooltip_text,
+					  GtkWidget ** button)
+{
+	GtkWidget *hbox;
+	GtkWidget *lbl;
+	GtkWidget *close_image;
+
+	hbox = gtk_hbox_new(FALSE, 4);
+	lbl = gtk_label_new(label_text);
+	gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 0);
+
+	close_image =
+	    gtk_image_new_from_stock(GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
+	*button = gtk_button_new();
+	gtk_button_set_relief(GTK_BUTTON(*button), GTK_RELIEF_NONE);
+	gtk_button_set_focus_on_click(GTK_BUTTON(*button), FALSE);
+	gtk_container_add(GTK_CONTAINER(*button), close_image);
+	gtk_widget_set_tooltip_text(*button, tooltip_text);
+	gtk_box_pack_start(GTK_BOX(hbox), *button, FALSE, FALSE, 0);
+
+	/* Set buttons name so the style will affect it */
+	gtk_widget_set_name(*button, "tab-with-close-button");
+
+	gtk_widget_show_all(hbox);
+	return hbox;
+}
+
+void build_frame(GtkWidget * parent, const gchar * title,
+		 GtkWidget * element, gboolean extend)
+{
+	/* vbox */
+	/*       label */
+	/*       hbox */
+	/*              fix */
+	/*              element */
+
+	GtkWidget *vbox;
+	GtkWidget *label;
+	GtkWidget *hbox;
+	GtkWidget *fix;
+	gchar *title_with_markup;
+
+	vbox = gtk_vbox_new(FALSE, 3);
+	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
+	gtk_box_pack_start(GTK_BOX(parent), vbox, extend, TRUE, 0);
+
+	label = gtk_label_new(NULL);
+	title_with_markup = g_strdup_printf("<b>%s</b>", title);
+	gtk_label_set_markup(GTK_LABEL(label), title_with_markup);
+	g_free(title_with_markup);
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+	gtk_widget_show(label);
+	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, TRUE, 0);
+
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
+
+	fix = gtk_fixed_new();
+	gtk_box_pack_start(GTK_BOX(hbox), fix, FALSE, TRUE, 6);
+
+	gtk_box_pack_start(GTK_BOX(hbox), element, TRUE, TRUE, 0);
+}
