@@ -53,4 +53,28 @@ void check_victory_points(GameParams * param, GtkWindow * main_window);
 
 extern UIDriver GTK_Driver;
 
+/** Prepare Gtk for close buttons on tabs.
+ *  Needs to be called once
+*/
+void prepare_gtk_for_close_button_on_tab(void);
+
+/** Create a label with a close button.
+ * @param label_text Text for the label
+ * @param tooltip_text Tooltip for the close button
+ * @retval button The close button
+ * @return Composite widget with label and close button
+ */
+GtkWidget *create_label_with_close_button(const gchar * label_text,
+					  const gchar * tooltip_text,
+					  GtkWidget ** button);
+
+/** Places a title above the element and adds the title and element to parent.
+ *  @param parent The parent to add the tital and element to.
+ *  @param title The title for the element.
+ *  @param element The element to add to parent.
+ *  @param extend True if element will take remaining space in parent.
+ */
+void build_frame(GtkWidget * parent, const gchar * title,
+		 GtkWidget * element, gboolean extend);
+
 #endif				/* __common_gtk_h */
