@@ -1,4 +1,4 @@
-/* Pioneers - Implementation of the excellent Settlers of Catan board game.
+/* Pioneers - Implementation of the excellent Settlers of Catan board game
  *   Go buy a copy.
  *
  * Copyright (C) 2007 Giancarlo Capella <giancarlo@comm.cc>
@@ -22,9 +22,11 @@
 #include "config.h"
 #include "colors.h"
 #include "player-icon.h"
+#include "game.h"
 #include <gtk/gtk.h>
 #include <string.h>
 #include <stdlib.h>
+#include <math.h>
 
 static gboolean load_pixbuf(const gchar * name, GdkPixbuf ** pixbuf);
 static void replace_colors(GdkPixbuf * pixbuf,
@@ -129,16 +131,13 @@ static void replace_colors(GdkPixbuf * pixbuf,
 }
 
 GdkPixbuf *playericon_create_icon(GtkWidget * widget, const gchar * style,
-				  GdkColor * color, gboolean viewer,
+				  GdkColor * color, gboolean spectator,
 				  gboolean connected, gboolean double_size)
 {
 	gint width, height;
 	GdkPixbuf *basic_image, *overlay_image, *scaled_image;
-	gchar **style_parts;
 	gboolean basic_substitute;
-
-	/* Determine the style for the player/viewer */
-	style_parts = g_strsplit(style, " ", 0);
+	PlayerType player_type;
 
 	gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
 	if (double_size) {
@@ -147,83 +146,101 @@ GdkPixbuf *playericon_create_icon(GtkWidget * widget, const gchar * style,
 	}
 	basic_image = NULL;
 	overlay_image = NULL;
-	if (!strcmp(style_parts[0], "ai")) {
+	player_type = determine_player_type(style);
+	/* Human players are allowed to have the square icon */
+	if (player_type == PLAYER_HUMAN
+	    && !strcmp(style, default_player_style)) {
+		player_type = PLAYER_UNKNOWN;
+	}
+
+	switch (player_type) {
+	case PLAYER_COMPUTER:
 		/* This is an AI */
 		basic_image = gdk_pixbuf_copy(ai_avatar);
 		basic_substitute = TRUE;
 		/** @todo RC 2007-07-17 For now, the AI does not have different appearances */
-	} else if (!strcmp(style_parts[0], "human")) {
-		/* Draw a bust */
-		gint variant;
-		GdkColor face_color, variant_color;
+		break;
+	case PLAYER_HUMAN:{
+			/* Draw a bust */
+			gint variant;
+			GdkColor face_color, variant_color;
 
-		playericon_parse_human_style(style, &face_color, &variant,
-					     &variant_color);
-		basic_image = gdk_pixbuf_copy(player_avatar.base);
-		basic_substitute = TRUE;
+			playericon_parse_human_style(style, &face_color,
+						     &variant,
+						     &variant_color);
+			basic_image = gdk_pixbuf_copy(player_avatar.base);
+			basic_substitute = TRUE;
 
-		/* Substitute the pure red face with the face color */
-		replace_colors(basic_image, &red, &face_color);
+			/* Substitute the pure red face with the face color */
+			replace_colors(basic_image, &red, &face_color);
 
-		/* Substitute the pure blue base with the variant color */
-		overlay_image =
-		    gdk_pixbuf_copy(g_slist_nth
-				    (player_avatar.variation,
-				     variant)->data);
-		replace_colors(overlay_image, &blue, &variant_color);
-	} else {
-		/* Unknown or square */
-		GdkGC *gc;
-		GdkPixmap *pixmap;
+			/* Substitute the pure blue base with the variant color */
+			overlay_image =
+			    gdk_pixbuf_copy(g_slist_nth
+					    (player_avatar.variation,
+					     variant)->data);
+			replace_colors(overlay_image, &blue,
+				       &variant_color);
+			break;
+		}
+	default:{
+			/* Unknown or square */
+			cairo_t *cr;
+			GdkPixmap *pixmap;
 
-		pixmap =
-		    gdk_pixmap_new(widget->window, width, height,
-				   gdk_visual_get_system()->depth);
-		gc = gdk_gc_new(pixmap);
-		if (viewer) {
-			GdkPixbuf *tmp;
-			/* Viewers have a transparent icon */
-			gdk_gc_set_foreground(gc, &black);
-			gdk_draw_rectangle(pixmap, gc, TRUE, 0, 0, width,
-					   height);
+			pixmap =
+			    gdk_pixmap_new(widget->window, width, height,
+					   gdk_visual_get_system()->depth);
+			cr = gdk_cairo_create(pixmap);
+			if (spectator) {
+				GdkPixbuf *tmp;
+				/* Spectators have a transparent icon */
+				gdk_cairo_set_source_color(cr, &black);
+				cairo_rectangle(cr, 0, 0, width, height);
+				cairo_fill(cr);
 
-			tmp =
-			    gdk_pixbuf_get_from_drawable(NULL, pixmap,
-							 NULL, 0, 0, 0, 0,
-							 -1, -1);
-			basic_image =
-			    gdk_pixbuf_add_alpha(tmp, TRUE, 0, 0, 0);
-		} else {
-			gdk_gc_set_foreground(gc, &black);
-			gdk_draw_rectangle(pixmap, gc, TRUE, 0, 0, width,
-					   height);
+				tmp =
+				    gdk_pixbuf_get_from_drawable(NULL,
+								 pixmap,
+								 NULL, 0,
+								 0, 0, 0,
+								 -1, -1);
+				basic_image =
+				    gdk_pixbuf_add_alpha(tmp, TRUE, 0, 0,
+							 0);
+			} else {
+				gdk_cairo_set_source_color(cr, color);
+				cairo_rectangle(cr, 0, 0, width, height);
+				cairo_fill(cr);
 
-			gdk_gc_set_foreground(gc, color);
-			gdk_draw_rectangle(pixmap, gc, TRUE, 1, 1,
-					   width - 2, height - 2);
-
-			if (!connected) {
-				gdk_gc_set_foreground(gc, &black);
-				gdk_draw_rectangle(pixmap, gc, FALSE,
-						   3, 3, width - 7,
-						   height - 7);
-				gdk_draw_rectangle(pixmap, gc, FALSE, 6, 6,
-						   width - 13,
-						   height - 13);
-				/* Don't draw the other emblem */
-				connected = TRUE;
+				gdk_cairo_set_source_color(cr, &black);
+				cairo_set_line_width(cr, 1.0);
+				cairo_rectangle(cr, 0.5, 0.5, width - 1,
+						height - 1);
+				cairo_stroke(cr);
+				if (!connected) {
+					cairo_rectangle(cr, 3.5, 3.5,
+							width - 7,
+							height - 7);
+					cairo_rectangle(cr, 6.5, 6.5,
+							width - 13,
+							height - 13);
+					cairo_stroke(cr);
+					/* Don't draw the other emblem */
+					connected = TRUE;
+				}
+				basic_image =
+				    gdk_pixbuf_get_from_drawable(NULL,
+								 pixmap,
+								 NULL, 0,
+								 0, 0, 0,
+								 -1, -1);
 			}
-
-			basic_image =
-			    gdk_pixbuf_get_from_drawable(NULL, pixmap,
-							 NULL, 0, 0, 0, 0,
-							 -1, -1);
+			basic_substitute = FALSE;
+			cairo_destroy(cr);
+			g_object_unref(pixmap);
 		}
-		basic_substitute = FALSE;
-		g_object_unref(gc);
-		g_object_unref(pixmap);
 	}
-	g_strfreev(style_parts);
 
 	if (basic_image && basic_substitute) {
 		/* Substitute the pure blue base with the player color */
@@ -245,25 +262,30 @@ GdkPixbuf *playericon_create_icon(GtkWidget * widget, const gchar * style,
 	g_object_unref(basic_image);
 
 	if (!connected) {
-		GdkGC *gc;
+		cairo_t *cr;
 		GdkPixmap *pixmap;
 		GdkPixbuf *tmp1, *tmp2;
 
 		pixmap =
 		    gdk_pixmap_new(widget->window, width, height,
 				   gdk_visual_get_system()->depth);
-		gc = gdk_gc_new(pixmap);
+		cr = gdk_cairo_create(pixmap);
 
-		gdk_gc_set_foreground(gc, &black);	/* Black will become transparent */
-		gdk_draw_rectangle(pixmap, gc, TRUE, 0, 0, width, height);
+		/* Black will become transparent */
+		gdk_cairo_set_source_color(cr, &black);
+		cairo_rectangle(cr, 0, 0, width, height);
+		cairo_fill(cr);
 
-		gdk_gc_set_foreground(gc, &red);
-		gdk_draw_arc(pixmap, gc, TRUE, width / 2, 0, width / 2,
-			     height / 2, 0, 360 * 64);
-		gdk_gc_set_foreground(gc, &white);
-		gdk_draw_rectangle(pixmap, gc, TRUE,
-				   width / 2 + 2, height / 4 - 1,
-				   width / 2 - 4, height / 8);
+		gdk_cairo_set_source_color(cr, &red);
+		cairo_move_to(cr, width / 2, height / 2);
+		cairo_arc(cr, width * 3 / 4, height / 4, width / 4, 0.0,
+			  2 * M_PI);
+		cairo_fill(cr);
+		gdk_cairo_set_source_color(cr, &white);
+		cairo_rectangle(cr,
+				width / 2 + 2, height / 4 - 1,
+				width / 2 - 4, height / 8);
+		cairo_fill(cr);
 
 		tmp1 =
 		    gdk_pixbuf_get_from_drawable(NULL, pixmap, NULL, 0, 0,
@@ -274,7 +296,7 @@ GdkPixbuf *playericon_create_icon(GtkWidget * widget, const gchar * style,
 				     GDK_INTERP_NEAREST, 255);
 		g_object_unref(tmp2);
 		g_object_unref(tmp1);
-		g_object_unref(gc);
+		cairo_destroy(cr);
 		g_object_unref(pixmap);
 	}
 	return scaled_image;
@@ -306,7 +328,7 @@ gboolean playericon_parse_human_style(const gchar * style,
 	gchar **style_parts;
 	gboolean parse_ok;
 
-	/* Determine the style for the player/viewer */
+	/* Determine the style for the player/spectator */
 	style_parts = g_strsplit(style, " ", 0);
 	parse_ok = FALSE;
 	if (!strcmp(style_parts[0], "human")) {
