@@ -29,6 +29,7 @@
 #include "game.h"
 #include "theme.h"
 #include "config-gnome.h"
+#include "gtkcompat.h"
 
 /*
 
@@ -298,8 +299,8 @@ static gboolean theme_initialize(MapTheme * t)
 		t->scaledata[i].native_image = pixbuf_copy;
 		t->scaledata[i].native_width =
 		    gdk_pixbuf_get_width(pixbuf);
-		t->scaledata[i].aspect =
-		    (float) gdk_pixbuf_get_width(pixbuf) /
+		t->scaledata[i].aspect = 1.0 *
+		    gdk_pixbuf_get_width(pixbuf) /
 		    gdk_pixbuf_get_height(pixbuf);
 		gdk_pixbuf_render_pixmap_and_mask(pixbuf,
 						  &(t->terrain_tiles[i]),
@@ -315,11 +316,11 @@ static gboolean theme_initialize(MapTheme * t)
 			if (theme_load_pixmap
 			    (t->port_tile_names[i], t->name, &pixbuf,
 			     &(t->port_tiles[i]), NULL)) {
-				gdk_drawable_get_size(t->port_tiles[i],
-						      &t->port_tiles_width
-						      [i],
-						      &t->port_tiles_height
-						      [i]);
+				gdk_pixmap_get_size(t->port_tiles[i],
+						    &t->port_tiles_width
+						    [i],
+						    &t->port_tiles_height
+						    [i]);
 				g_object_unref(pixbuf);
 			}
 		} else
@@ -699,3 +700,37 @@ void theme_register_callback(GCallback callback)
 {
 	callback_list = g_list_append(callback_list, callback);
 }
+
+GdkPixmap *theme_get_terrain_pixmap(Terrain terrain)
+{
+	return theme_get_current()->terrain_tiles[terrain];
+}
+
+gint expose_terrain_cb(GtkWidget * area,
+		       G_GNUC_UNUSED GdkEventExpose * event,
+		       gpointer terraindata)
+{
+	MapTheme *theme = theme_get_current();
+	cairo_t *cr;
+	GdkPixbuf *p;
+	gint height;
+	Terrain terrain = GPOINTER_TO_INT(terraindata);
+
+	if (area->window == NULL)
+		return FALSE;
+
+	cr = gdk_cairo_create(area->window);
+
+	height = area->allocation.width / theme->scaledata[terrain].aspect;
+	p = gdk_pixbuf_scale_simple(theme->scaledata[terrain].native_image,
+				    area->allocation.width, height,
+				    GDK_INTERP_BILINEAR);
+
+	gdk_cairo_set_source_pixbuf(cr, p, 0, 0);
+	cairo_rectangle(cr, 0, 0, area->allocation.width, height);
+	cairo_fill(cr);
+
+	g_object_unref(p);
+	cairo_destroy(cr);
+	return FALSE;
+}
