@@ -63,8 +63,22 @@ static GtkWidget *roll_numbers[12 + 1];
 static GtkCheckMenuItem *shuffle_tile;
 static GtkWidget *port_menu;
 static GtkWidget *port_directions[6];
-static GtkWidget *hresize_buttons[2];
-static GtkWidget *vresize_buttons[2];
+static GtkWidget *hresize_buttons[4];
+static GtkWidget *vresize_buttons[4];
+typedef enum {
+	RESIZE_INSERT_LEFT,
+	RESIZE_REMOVE_LEFT,
+	RESIZE_INSERT_RIGHT,
+	RESIZE_REMOVE_RIGHT
+} hresize_type;
+/* order of vresize must match order of hresize */
+typedef enum {
+	RESIZE_INSERT_TOP,
+	RESIZE_REMOVE_TOP,
+	RESIZE_INSERT_BOTTOM,
+	RESIZE_REMOVE_BOTTOM
+} vresize_type;
+
 static GuiMap *gmap;
 static Hex *current_hex;
 
@@ -143,27 +157,24 @@ static void error_dialog(const char *fmt, ...)
 	gtk_widget_destroy(dialog);
 }
 
-static void clear_hex(Hex * hex)
-{
-	g_return_if_fail(hex != NULL);
-	hex->terrain = LAST_TERRAIN;
-	hex->resource = NO_RESOURCE;
-	hex->chit_pos = -1;
-	hex->roll = 0;
-	hex->shuffle = TRUE;
-}
-
 static void fill_map(Map * map)
 {
 	gint x, y;
 
 	for (y = 0; y < map->y_size; y++) {
 		for (x = 0; x < map->x_size; x++) {
-			if (x == map->x_size - 1 && y % 2 == 1)
+			if (x == 0 && y % 2 == 0 && map->shrink_left) {
 				continue;
-			if (map->grid[y][x] != NULL)
+			}
+			if (x == map->x_size - 1 && y % 2 == 1
+			    && map->shrink_right) {
 				continue;
-			clear_hex(map_add_hex(map, x, y));
+			}
+			if (map->grid[y][x] != NULL) {
+				continue;
+			}
+			/* Add a default hex on the empty spot */
+			map_reset_hex(map, x, y);
 		}
 	}
 }
@@ -204,34 +215,55 @@ static gboolean terrain_has_chit(Terrain terrain)
 	return FALSE;
 }
 
-static void build_map_resize(GtkWidget * table, gint row, gint col,
+static void build_map_resize(GtkWidget * table, guint col, guint row,
 			     GtkOrientation dir, GtkWidget ** buttons,
 			     GCallback resize_callback)
 {
-	static const char *symbols[] = { "+", "--" };
-	static gint values[] = { +1, -1 };
+	/* symbols[] must match order of hresize_type and vresize_type; */
+	static const gchar *symbols[] =
+	    { GTK_STOCK_ADD, GTK_STOCK_REMOVE, GTK_STOCK_ADD,
+		GTK_STOCK_REMOVE
+	};
+	/* The order must match hresize_type and vresize_type, and also depends on the orientation */
+	static const gchar *tooltip[] = {
+		N_("Insert a row"),
+		N_("Delete a row"),
+		N_("Insert a column"),
+		N_("Delete a column")
+	};
+
 	GtkWidget *box;
 	gint i;
 
-	if (dir == GTK_ORIENTATION_VERTICAL)
+	if (dir == GTK_ORIENTATION_VERTICAL) {
 		box = gtk_vbox_new(FALSE, 0);
-	else
+	} else {
 		box = gtk_hbox_new(FALSE, 0);
-
-	gtk_box_pack_start(GTK_BOX(box), gtk_fixed_new(), TRUE, TRUE, 0);
-
-	for (i = 0; i < 2; i++) {
-		buttons[i] = gtk_button_new_with_label(symbols[i]);
-		gtk_box_pack_start(GTK_BOX(box), buttons[i], FALSE, TRUE,
-				   0);
-		g_signal_connect(G_OBJECT(buttons[i]), "clicked",
-				 resize_callback,
-				 GINT_TO_POINTER(values[i]));
 	}
 
+	for (i = 0; i < 4; i++) {
+		buttons[i] =
+		    GTK_WIDGET(gtk_tool_button_new_from_stock(symbols[i]));
+		gtk_tool_item_set_tooltip_text(GTK_TOOL_ITEM(buttons[i]),
+					       _(tooltip
+						 [i % 2 +
+						  (dir ==
+						   GTK_ORIENTATION_VERTICAL
+						   ? 0 : 2)]));
+
+		if (i < 2) {
+			gtk_box_pack_start(GTK_BOX(box), buttons[i],
+					   FALSE, TRUE, 0);
+		} else {
+			gtk_box_pack_end(GTK_BOX(box), buttons[i],
+					 FALSE, TRUE, 0);
+		}
+		g_signal_connect(G_OBJECT(buttons[i]), "clicked",
+				 resize_callback, GINT_TO_POINTER(i));
+	}
 	gtk_box_pack_start(GTK_BOX(box), gtk_fixed_new(), TRUE, TRUE, 0);
 
-	gtk_table_attach(GTK_TABLE(table), box, row, row + 1, col, col + 1,
+	gtk_table_attach(GTK_TABLE(table), box, col, col + 1, row, row + 1,
 			 GTK_FILL, GTK_FILL, 0, 0);
 }
 
@@ -256,14 +288,42 @@ static gint button_press_map_cb(GtkWidget * area, GdkEventButton * event,
 	if (area->window == NULL || gmap->map == NULL)
 		return FALSE;
 
+	if (event->button != 1)
+		return TRUE;
+
 	current_hex = guimap_find_hex(gmap, event->x, event->y);
 	if (current_hex == NULL)
 		return TRUE;
 
 	menu = NULL;
-	if (event->button == 1)
-		menu = terrain_menu;
-	else if (current_hex->roll > 0) {
+	if (event->button == 1) {
+		MapElement element;
+		Node *current_node;
+		gint distance_node;
+		gint distance_hex;
+
+		current_node = guimap_find_node(gmap, event->x, event->y);
+		element.node = current_node;
+		distance_node =
+		    guimap_distance_cursor(gmap, &element, MAP_NODE,
+					   event->x, event->y);
+		element.hex = current_hex;
+		distance_hex =
+		    guimap_distance_cursor(gmap, &element, MAP_HEX,
+					   event->x, event->y);
+
+		if (distance_node < distance_hex) {
+			current_node->no_setup = !current_node->no_setup;
+			for (i = 0; i < G_N_ELEMENTS(current_node->hexes);
+			     i++) {
+				guimap_draw_hex(gmap,
+						current_node->hexes[i]);
+			}
+			return TRUE;
+		} else {
+			menu = terrain_menu;
+		}
+	} else if (current_hex->roll > 0) {
 		menu = roll_menu;
 		for (i = 2; i <= 12; i++) {
 			if (roll_numbers[i] != NULL)
@@ -356,12 +416,36 @@ static gint key_press_map_cb(GtkWidget * area, GdkEventKey * event,
 	return TRUE;
 }
 
-static void post_change(gint * size, GtkWidget ** buttons, gint amt)
+static void update_resize_buttons(void)
 {
-	*size += amt;
-	gtk_widget_set_sensitive(buttons[0], *size < MAP_SIZE);
-	gtk_widget_set_sensitive(buttons[1], *size > 1);
-	fill_map(gmap->map);
+	gtk_widget_set_sensitive(hresize_buttons[RESIZE_REMOVE_LEFT],
+				 gmap->map->x_size > 1 ||
+				 !(gmap->map->shrink_right ||
+				   gmap->map->shrink_left));
+	gtk_widget_set_sensitive(hresize_buttons[RESIZE_REMOVE_RIGHT],
+				 gmap->map->x_size > 1 ||
+				 !(gmap->map->shrink_right ||
+				   gmap->map->shrink_left));
+	gtk_widget_set_sensitive(hresize_buttons[RESIZE_INSERT_LEFT],
+				 gmap->map->x_size < MAP_SIZE ||
+				 gmap->map->shrink_left);
+	gtk_widget_set_sensitive(hresize_buttons[RESIZE_INSERT_RIGHT],
+				 gmap->map->x_size < MAP_SIZE ||
+				 gmap->map->shrink_right);
+
+	gtk_widget_set_sensitive(vresize_buttons[RESIZE_REMOVE_TOP],
+				 gmap->map->y_size > 1 &&
+				 (gmap->map->x_size < MAP_SIZE ||
+				  gmap->map->shrink_left));
+	gtk_widget_set_sensitive(vresize_buttons[RESIZE_REMOVE_BOTTOM],
+				 gmap->map->y_size > 1);
+	gtk_widget_set_sensitive(vresize_buttons[RESIZE_INSERT_TOP],
+				 gmap->map->y_size < MAP_SIZE
+				 && (gmap->map->x_size < MAP_SIZE
+				     || gmap->map->shrink_left));
+	gtk_widget_set_sensitive(vresize_buttons[RESIZE_INSERT_BOTTOM],
+				 gmap->map->y_size < MAP_SIZE);
+
 	scale_map(gmap);
 	guimap_display(gmap);
 }
@@ -369,31 +453,50 @@ static void post_change(gint * size, GtkWidget ** buttons, gint amt)
 static void change_height(G_GNUC_UNUSED GtkWidget * menu,
 			  gpointer user_data)
 {
-	if (GPOINTER_TO_INT(user_data) < 0) {
-		gint x;
-		for (x = 0; x < gmap->map->x_size; x++)
-			clear_hex(gmap->map->
-				  grid[gmap->map->y_size - 1][x]);
+	switch (GPOINTER_TO_INT(user_data)) {
+	case RESIZE_REMOVE_BOTTOM:
+		map_modify_row_count(gmap->map, MAP_MODIFY_REMOVE,
+				     MAP_MODIFY_ROW_BOTTOM);
+		break;
+	case RESIZE_INSERT_BOTTOM:
+		map_modify_row_count(gmap->map, MAP_MODIFY_INSERT,
+				     MAP_MODIFY_ROW_BOTTOM);
+		break;
+	case RESIZE_REMOVE_TOP:
+		map_modify_row_count(gmap->map, MAP_MODIFY_REMOVE,
+				     MAP_MODIFY_ROW_TOP);
+		break;
+	case RESIZE_INSERT_TOP:
+		map_modify_row_count(gmap->map, MAP_MODIFY_INSERT,
+				     MAP_MODIFY_ROW_TOP);
+		break;
 	}
-	post_change(&gmap->map->y_size, vresize_buttons,
-		    GPOINTER_TO_INT(user_data));
+	update_resize_buttons();
 }
 
 static void change_width(G_GNUC_UNUSED GtkWidget * menu,
 			 gpointer user_data)
 {
-	if (GPOINTER_TO_INT(user_data) < 0) {
-		gint x, y;
-		for (y = 0; y < gmap->map->y_size; y++) {
-			if (y % 2 == 0)
-				x = gmap->map->x_size - 1;
-			else
-				x = gmap->map->x_size - 2;
-			clear_hex(gmap->map->grid[y][x]);
-		}
+	switch (GPOINTER_TO_INT(user_data)) {
+	case RESIZE_REMOVE_RIGHT:
+		map_modify_column_count(gmap->map, MAP_MODIFY_REMOVE,
+					MAP_MODIFY_COLUMN_RIGHT);
+		break;
+	case RESIZE_INSERT_RIGHT:
+		map_modify_column_count(gmap->map, MAP_MODIFY_INSERT,
+					MAP_MODIFY_COLUMN_RIGHT);
+		break;
+	case RESIZE_REMOVE_LEFT:
+		map_modify_column_count(gmap->map, MAP_MODIFY_REMOVE,
+					MAP_MODIFY_COLUMN_LEFT);
+		break;
+	case RESIZE_INSERT_LEFT:
+		map_modify_column_count(gmap->map, MAP_MODIFY_INSERT,
+					MAP_MODIFY_COLUMN_LEFT);
+		break;
 	}
-	post_change(&gmap->map->x_size, hresize_buttons,
-		    GPOINTER_TO_INT(user_data));
+	update_resize_buttons();
+
 }
 
 static GtkWidget *build_map(void)
@@ -401,13 +504,13 @@ static GtkWidget *build_map(void)
 	GtkWidget *table;
 	GtkWidget *area;
 
-	table = gtk_table_new(2, 2, FALSE);
+	table = gtk_table_new(4, 2, FALSE);
 
 	gmap = guimap_new();
 	guimap_set_show_no_setup_nodes(gmap, TRUE);
 	area = guimap_build_drawingarea(gmap, MAP_WIDTH, MAP_HEIGHT);
 
-	GTK_WIDGET_SET_FLAGS(area, GTK_CAN_FOCUS);
+	gtk_widget_set_can_focus(area, TRUE);
 	gtk_widget_add_events(gmap->area, GDK_ENTER_NOTIFY_MASK
 			      | GDK_BUTTON_PRESS_MASK
 			      | GDK_KEY_PRESS_MASK);
@@ -417,13 +520,13 @@ static GtkWidget *build_map(void)
 			 G_CALLBACK(button_press_map_cb), gmap);
 	g_signal_connect(G_OBJECT(gmap->area), "key_press_event",
 			 G_CALLBACK(key_press_map_cb), gmap);
-	gtk_table_attach(GTK_TABLE(table), gmap->area, 0, 1, 0, 1,
+	gtk_table_attach(GTK_TABLE(table), gmap->area, 0, 1, 2, 3,
 			 GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0,
 			 0);
 
-	build_map_resize(table, 0, 1, GTK_ORIENTATION_HORIZONTAL,
+	build_map_resize(table, 1, 2, GTK_ORIENTATION_VERTICAL,
 			 vresize_buttons, G_CALLBACK(change_height));
-	build_map_resize(table, 1, 0, GTK_ORIENTATION_VERTICAL,
+	build_map_resize(table, 0, 3, GTK_ORIENTATION_HORIZONTAL,
 			 hresize_buttons, G_CALLBACK(change_width));
 
 	return table;
@@ -473,8 +576,10 @@ static GtkWidget *build_terrain_menu(void)
 	gint i;
 	GtkWidget *menu;
 	GtkWidget *item;
-	GdkPixmap *pixmap;
+	GdkPixbuf *pixbuf;
 	GtkWidget *image;
+	gint width;
+	gint height;
 	MapTheme *theme = theme_get_current();
 
 	menu = gtk_menu_new();
@@ -490,11 +595,24 @@ static GtkWidget *build_terrain_menu(void)
 				 G_CALLBACK(select_terrain_cb),
 				 GINT_TO_POINTER(i));
 
-		pixmap = theme->terrain_tiles[i];
-		if (i == LAST_TERRAIN || pixmap == NULL)
+		if (i == LAST_TERRAIN)
 			continue;
 
-		image = gtk_image_new_from_pixmap(pixmap, NULL);
+		/* Use the default size, or smaller */
+		gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
+		if (height > width / theme->scaledata[i].aspect) {
+			height = width / theme->scaledata[i].aspect;
+		} else if (width > height * theme->scaledata[i].aspect) {
+			width = height * theme->scaledata[i].aspect;
+		}
+
+		pixbuf =
+		    gdk_pixbuf_scale_simple(theme->
+					    scaledata[i].native_image,
+					    width, height,
+					    GDK_INTERP_BILINEAR);
+
+		image = gtk_image_new_from_pixbuf(pixbuf);
 		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
 					      image);
 	}
@@ -558,6 +676,7 @@ static GtkWidget *build_roll_menu(void)
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu),
 			      gtk_separator_menu_item_new());
 
+	/* Menu item */
 	item = gtk_check_menu_item_new_with_label(_("Shuffle"));
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 	g_signal_connect(G_OBJECT(item), "toggled",
@@ -649,53 +768,80 @@ static GtkWidget *build_port_menu(void)
 	return menu;
 }
 
-static GtkWidget *build_frame(GtkWidget * parent, const gchar * title,
-			      GtkWidget * element)
-{
-	GtkWidget *frame;
-	GtkWidget *vbox;
-
-	frame = gtk_frame_new(title);
-	gtk_box_pack_start(GTK_BOX(parent), frame, FALSE, TRUE, 0);
-
-	vbox = gtk_vbox_new(FALSE, 3);
-	gtk_container_add(GTK_CONTAINER(frame), vbox);
-	gtk_container_set_border_width(GTK_CONTAINER(vbox), 3);
-
-	gtk_box_pack_start(GTK_BOX(vbox), element, FALSE, TRUE, 0);
-
-	return frame;
-}
-
 static GtkWidget *build_settings(GtkWindow * main_window)
 {
-	GtkWidget *vbox, *hbox, *lvbox, *rvbox;
+	/* vbox */
+	/*      outer_hbox */
+	/*              inner_hbox */
+	/*                      lvbox */
+	/*                      fix */
+	/*              inner_hbox */
+	/*                      vsep */
+	/*                      rvbox */
 
+	GtkWidget *vbox;
+	GtkWidget *outer_hbox;
+	GtkWidget *inner_hbox;
+	GtkWidget *lvbox;
+	GtkWidget *fix;
+	GtkWidget *vsep;
+	GtkWidget *rvbox;
+
+
+	/* vbox */
 	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
 
-	hbox = gtk_hbox_new(TRUE, 10);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
+	/*outer_hbox */
+	outer_hbox = gtk_hbox_new(TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), outer_hbox, TRUE, TRUE, 0);
 
-	lvbox = gtk_vbox_new(FALSE, 5);
-	gtk_box_pack_start(GTK_BOX(hbox), lvbox, FALSE, TRUE, 0);
+	/* inner_hbox */
+	inner_hbox = gtk_hbox_new(FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(outer_hbox), inner_hbox, TRUE, TRUE, 0);
 
-	rvbox = gtk_vbox_new(FALSE, 5);
-	gtk_box_pack_start(GTK_BOX(hbox), rvbox, FALSE, TRUE, 0);
+	/* lvbox */
+	lvbox = gtk_vbox_new(FALSE, 10);
+	gtk_box_pack_start(GTK_BOX(inner_hbox), lvbox, TRUE, TRUE, 0);
 
+	/* fix */
+	fix = gtk_fixed_new();
+	gtk_box_pack_start(GTK_BOX(inner_hbox), fix, FALSE, TRUE, 0);
+
+	/* inner_hbox */
+	inner_hbox = gtk_hbox_new(FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(outer_hbox), inner_hbox, TRUE, TRUE, 0);
+
+	/* vsep */
+	vsep = gtk_vseparator_new();
+	gtk_widget_show(vsep);
+	gtk_box_pack_start(GTK_BOX(inner_hbox), vsep, FALSE, FALSE, 0);
+
+	/* rvbox */
+	rvbox = gtk_vbox_new(FALSE, 10);
+	gtk_box_pack_start(GTK_BOX(inner_hbox), rvbox, TRUE, TRUE, 0);
+
+	/* get elements */
 	game_settings = GAMESETTINGS(game_settings_new(TRUE));
 	game_rules = GAMERULES(game_rules_new());
 	game_resources = GAMERESOURCES(game_resources_new());
 	game_devcards = GAMEDEVCARDS(game_devcards_new());
 	game_buildings = GAMEBUILDINGS(game_buildings_new());
 
-	build_frame(lvbox, _("Game Parameters"),
-		    GTK_WIDGET(game_settings));
-	build_frame(lvbox, _("Rules"), GTK_WIDGET(game_rules));
-	build_frame(lvbox, _("Resources"), GTK_WIDGET(game_resources));
-	build_frame(rvbox, _("Buildings"), GTK_WIDGET(game_buildings));
-	build_frame(rvbox, _("Development Cards"),
-		    GTK_WIDGET(game_devcards));
+	/* Caption */
+	build_frame(lvbox, _("Game parameters"),
+		    GTK_WIDGET(game_settings), FALSE);
+	/* Caption */
+	build_frame(lvbox, _("Rules"), GTK_WIDGET(game_rules), FALSE);
+	/* Caption */
+	build_frame(lvbox, _("Resources"), GTK_WIDGET(game_resources),
+		    FALSE);
+	/* Caption */
+	build_frame(rvbox, _("Buildings"), GTK_WIDGET(game_buildings),
+		    FALSE);
+	/* Caption */
+	build_frame(rvbox, _("Development cards"),
+		    GTK_WIDGET(game_devcards), FALSE);
 
 	g_signal_connect(G_OBJECT(game_settings), "check",
 			 G_CALLBACK(check_vp_cb), main_window);
@@ -711,6 +857,7 @@ static void set_window_title(const gchar * title)
 		window_title = NULL;
 	} else
 		window_title = g_strdup(title);
+	/* Application caption */
 	str = g_strdup_printf("%s: %s", _("Pioneers Editor"), title);
 
 	gtk_window_set_title(GTK_WINDOW(toplevel), str);
@@ -788,6 +935,7 @@ static void load_game(const gchar * file, gboolean is_reload)
 	const gchar *gamefile;
 	GameParams *new_params;
 	gchar *new_filename;
+	gint i;
 
 	if (file == NULL)
 		gamefile = default_game;
@@ -805,8 +953,16 @@ static void load_game(const gchar * file, gboolean is_reload)
 		new_params->title = g_strdup("Untitled");
 		map_free(new_params->map);
 		new_params->map = map_new();
-		new_params->map->x_size = 6;
-		new_params->map->y_size = 6;
+		for (i = 0; i < 6; i++) {
+			map_modify_row_count(new_params->map,
+					     MAP_MODIFY_INSERT,
+					     MAP_MODIFY_ROW_BOTTOM);
+		}
+		for (i = 0; i < 11; i++) {
+			map_modify_column_count(new_params->map,
+						MAP_MODIFY_INSERT,
+						MAP_MODIFY_COLUMN_RIGHT);
+		}
 		new_params->map->chits =
 		    g_array_new(FALSE, FALSE, sizeof(gint));
 		new_filename = NULL;
@@ -829,6 +985,7 @@ static void load_game(const gchar * file, gboolean is_reload)
 		scale_map(gmap);
 		guimap_display(gmap);
 	}
+	update_resize_buttons();
 }
 
 static void save_game(const gchar * file)
@@ -851,13 +1008,15 @@ static void load_game_menu_cb(void)
 {
 	GtkWidget *dialog;
 
-	dialog = gtk_file_chooser_dialog_new(_("Open Game"),
-					     GTK_WINDOW(toplevel),
-					     GTK_FILE_CHOOSER_ACTION_OPEN,
-					     GTK_STOCK_CANCEL,
-					     GTK_RESPONSE_CANCEL,
-					     GTK_STOCK_OPEN,
-					     GTK_RESPONSE_OK, NULL);
+	dialog = gtk_file_chooser_dialog_new(
+						    /* Dialog caption */
+						    _("Open Game"),
+						    GTK_WINDOW(toplevel),
+						    GTK_FILE_CHOOSER_ACTION_OPEN,
+						    GTK_STOCK_CANCEL,
+						    GTK_RESPONSE_CANCEL,
+						    GTK_STOCK_OPEN,
+						    GTK_RESPONSE_OK, NULL);
 	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog),
 				      default_game);
 	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK) {
@@ -877,13 +1036,16 @@ static void save_as_menu_cb(void)
 {
 	GtkWidget *dialog;
 
-	dialog = gtk_file_chooser_dialog_new(_("Save as..."),
-					     GTK_WINDOW(toplevel),
-					     GTK_FILE_CHOOSER_ACTION_SAVE,
-					     GTK_STOCK_CANCEL,
-					     GTK_RESPONSE_CANCEL,
-					     GTK_STOCK_SAVE,
-					     GTK_RESPONSE_ACCEPT, NULL);
+	dialog = gtk_file_chooser_dialog_new(
+						    /* Dialog caption */
+						    _("Save As..."),
+						    GTK_WINDOW(toplevel),
+						    GTK_FILE_CHOOSER_ACTION_SAVE,
+						    GTK_STOCK_CANCEL,
+						    GTK_RESPONSE_CANCEL,
+						    GTK_STOCK_SAVE,
+						    GTK_RESPONSE_ACCEPT,
+						    NULL);
 	gtk_dialog_set_default_response(GTK_DIALOG(dialog),
 					GTK_RESPONSE_ACCEPT);
 	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
@@ -912,14 +1074,16 @@ static void change_title_menu_cb(void)
 {
 	GtkWidget *dialog, *vbox, *hbox, *label, *entry;
 
-	dialog = gtk_dialog_new_with_buttons(_("Change Title"),
-					     GTK_WINDOW(toplevel),
-					     GTK_DIALOG_MODAL |
-					     GTK_DIALOG_DESTROY_WITH_PARENT,
-					     GTK_STOCK_CANCEL,
-					     GTK_RESPONSE_CANCEL,
-					     GTK_STOCK_OK,
-					     GTK_RESPONSE_OK, NULL);
+	dialog = gtk_dialog_new_with_buttons(
+						    /* Dialog caption */
+						    _("Change Title"),
+						    GTK_WINDOW(toplevel),
+						    GTK_DIALOG_MODAL |
+						    GTK_DIALOG_DESTROY_WITH_PARENT,
+						    GTK_STOCK_CANCEL,
+						    GTK_RESPONSE_CANCEL,
+						    GTK_STOCK_OK,
+						    GTK_RESPONSE_OK, NULL);
 	gtk_dialog_set_default_response(GTK_DIALOG(dialog),
 					GTK_RESPONSE_OK);
 	g_signal_connect(G_OBJECT(dialog), "destroy",
@@ -934,7 +1098,8 @@ static void change_title_menu_cb(void)
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
 
-	label = gtk_label_new(_("New Title:"));
+	/* Label */
+	label = gtk_label_new(_("New title:"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
 	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 
@@ -990,38 +1155,115 @@ static void about_menu_cb(void)
 		AUTHORLIST
 	};
 
-	aboutbox_display(_("Pioneers Game Editor"), authors);
+	/* About dialog caption */
+	aboutbox_display(_("About Pioneers Game Editor"), authors);
+}
+
+/** Toggles full screen mode.
+ * @param GtkToggleAction The calling action.
+ * @param main_window The window to toggle full screen mode.
+*/
+static void toggle_full_screen_cb(GtkToggleAction * caller,
+				  gpointer main_window)
+{
+	if (gtk_toggle_action_get_active(caller)) {
+		gtk_window_fullscreen(GTK_WINDOW(main_window));
+	} else {
+		gtk_window_unfullscreen(GTK_WINDOW(main_window));
+	}
+}
+
+static void zoom_normal_cb(void)
+{
+	guimap_zoom_normal(gmap);
+}
+
+static void zoom_center_map_cb(void)
+{
+	guimap_zoom_center_map(gmap);
 }
 
 static GtkActionEntry entries[] = {
-	{"FileMenu", NULL, N_("_File"), NULL, NULL, NULL},
-	{"HelpMenu", NULL, N_("_Help"), NULL, NULL, NULL},
+	{"FileMenu", NULL,
+	 /* Menu entry */
+	 N_("_File"), NULL, NULL, NULL},
+	{"ViewMenu", NULL,
+	 /* Menu entry */
+	 N_("_View"), NULL, NULL, NULL},
+	{"HelpMenu", NULL,
+	 /* Menu entry */
+	 N_("_Help"), NULL, NULL, NULL},
 
-	{"New", GTK_STOCK_NEW, N_("_New"), "<control>N",
+	{"New", GTK_STOCK_NEW,
+	 /* Menu entry */
+	 N_("_New"), "<control>N",
 	 N_("Create a new game"), new_game_menu_cb},
-	{"Open", GTK_STOCK_OPEN, N_("_Open..."), "<control>O",
+	{"Open", GTK_STOCK_OPEN,
+	 /* Menu entry */
+	 N_("_Open..."), "<control>O",
+	 /* Tooltip for Open menu entry */
 	 N_("Open an existing game"), load_game_menu_cb},
-	{"Save", GTK_STOCK_SAVE, N_("_Save"), "<control>S",
+	{"Save", GTK_STOCK_SAVE,
+	 /* Menu entry */
+	 N_("_Save"), "<control>S",
+	 /* Tooltip for Save menu entry */
 	 N_("Save game"), save_game_menu_cb},
-	{"SaveAs", GTK_STOCK_SAVE_AS, N_("Save _As..."),
+	{"SaveAs", GTK_STOCK_SAVE_AS,
+	 /* Menu entry */
+	 N_("Save _As..."),
 	 "<control><shift>S",
+	 /* Tooltip for Save As menu entry */
 	 N_("Save as"), save_as_menu_cb},
-	{"ChangeTitle", NULL, N_("_Change title"), "<control>T",
+	{"ChangeTitle", NULL,
+	 /* Menu entry */
+	 N_("_Change Title"), "<control>T",
+	 /* Tooltip for Change Title menu entry */
 	 N_("Change game title"), change_title_menu_cb},
-	{"CheckVP", GTK_STOCK_APPLY, N_("_Check Victory Point Target"),
+	{"CheckVP", GTK_STOCK_APPLY,
+	 /* Menu entry */
+	 N_("_Check Victory Point Target"),
 	 NULL,
+	 /* Tooltip for Check Victory Point Target menu entry */
 	 N_("Check whether the game can be won"), G_CALLBACK(check_vp_cb)},
-	{"Quit", GTK_STOCK_QUIT, N_("_Quit"), "<control>Q",
+	{"Quit", GTK_STOCK_QUIT,
+	 /* Menu entry */
+	 N_("_Quit"), "<control>Q",
+	 /* Tooltip for Quit menu entry */
 	 N_("Quit"), exit_cb},
 
+	{"Full", GTK_STOCK_ZOOM_FIT,
+	 /* Menu entry */
+	 N_("_Reset"),
+	 "<control>0",
+	 /* Tooltip for Reset menu entry */
+	 N_("View the full map"), zoom_normal_cb},
+	{"Center", NULL,
+	 /* Menu entry */
+	 N_("_Center"), NULL,
+	 /* Tooltip for Center menu entry */
+	 N_("Center the map"), zoom_center_map_cb},
+
 #ifdef HAVE_HELP
 	/* Disable this item, until the help is written
 	   {"Contents", GTK_STOCK_HELP, N_("_Contents"), "F1",
 	   N_("Contents"), contents_menu_cb},
 	 */
 #endif
-	{"About", NULL, N_("_About Pioneers Editor"), NULL,
-	 N_("Information about Pioneers Editor"), about_menu_cb}
+	{"About", NULL,
+	 /* Menu entry */
+	 N_("_About Pioneers Editor"), NULL,
+	 /* Tooltip for About Pioneers Editor menu entry */
+	 N_("Information about Pioneers Editor"), about_menu_cb},
+};
+
+static GtkToggleActionEntry toggle_entries[] = {
+	{"FullScreen", GTK_STOCK_FULLSCREEN,
+	 /* Menu entry */
+	 N_("_Fullscreen"),
+	 "<alt>Return",
+	 /* Tooltip for Fullscreen menu entry */
+	 N_("Set window to full screen mode"),
+	 G_CALLBACK(toggle_full_screen_cb), FALSE}
 };
 
 /* *INDENT-OFF* */
@@ -1039,6 +1281,11 @@ static const char *ui_description =
 "      <separator/>"
 "      <menuitem action='Quit'/>"
 "    </menu>"
+"    <menu action='ViewMenu'>"
+"      <menuitem action='FullScreen'/>"
+"      <menuitem action='Full'/>"
+"      <menuitem action='Center'/>"
+"    </menu>"
 "    <menu action='HelpMenu'>"
 #ifdef HAVE_HELP
 /* Disable this menu item, until the help is written
@@ -1091,8 +1338,8 @@ int main(int argc, char *argv[])
 	textdomain(PACKAGE);
 	bind_textdomain_codeset(PACKAGE, "UTF-8");
 
-	/* Long description in the command line: --help */
 	context =
+	    /* Long description in the command line: --help */
 	    g_option_context_new(_("- Editor for games of Pioneers"));
 	g_option_context_add_main_entries(context, commandline_entries,
 					  GETTEXT_PACKAGE);
@@ -1124,6 +1371,9 @@ int main(int argc, char *argv[])
 	gtk_action_group_set_translation_domain(action_group, PACKAGE);
 	gtk_action_group_add_actions(action_group, entries,
 				     G_N_ELEMENTS(entries), toplevel);
+	gtk_action_group_add_toggle_actions(action_group, toggle_entries,
+					    G_N_ELEMENTS(toggle_entries),
+					    toplevel);
 
 	ui_manager = gtk_ui_manager_new();
 	gtk_ui_manager_insert_action_group(ui_manager, action_group, 0);
@@ -1158,10 +1408,12 @@ int main(int argc, char *argv[])
 	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP);
 
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), build_map(),
+				 /* Tab page name */
 				 gtk_label_new(_("Map")));
 
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
 				 build_settings(GTK_WINDOW(toplevel)),
+				 /* Tab page name */
 				 gtk_label_new(_("Settings")));
 
 	terrain_menu = build_terrain_menu();
