@@ -40,6 +40,7 @@
 #include <sys/stat.h>
 #include <netdb.h>
 #include <time.h>
+#include <signal.h>
 
 #include <glib.h>
 #include "network.h"
@@ -85,6 +86,7 @@ struct _Client {
 static gchar *redirect_location = NULL;
 static gchar *myhostname = NULL;
 static gboolean can_create_games;
+static gboolean request_quit = FALSE;
 
 static int port_low = 0, port_high = 0;
 
@@ -97,8 +99,8 @@ static gint max_fd;
 
 /* Command line data */
 static gboolean make_daemon = FALSE;
-static const gchar *pidfile = NULL;
-static const gchar *port_range = NULL;
+static gchar *pidfile = NULL;
+static gchar *port_range = NULL;
 static gboolean enable_debug = FALSE;
 static gboolean enable_syslog_debug = FALSE;
 static gboolean show_version = FALSE;
@@ -386,6 +388,7 @@ static GList *load_game_desc(gchar * fname, GList * titles)
 			titles =
 			    g_list_insert_sorted(titles, g_strdup(title),
 						 (GCompareFunc) strcmp);
+			g_free(line);
 			break;
 		}
 		g_free(line);
@@ -423,13 +426,19 @@ static GList *load_game_types(void)
 	return titles;
 }
 
+/** Send the title and free the associated memory. */
+static void client_send_type(gpointer data, gpointer user_data)
+{
+	Client *client = user_data;
+
+	client_printf(client, "title=%s\n", data);
+	g_free(data);
+}
+
 static void client_list_types(Client * client)
 {
 	GList *list = load_game_types();
-
-	for (; list != NULL; list = g_list_next(list)) {
-		client_printf(client, "title=%s\n", list->data);
-	}
+	g_list_foreach(list, client_send_type, client);
 	g_list_free(list);
 }
 
@@ -522,6 +531,7 @@ static void client_create_new_server(Client * client, gchar * line)
 	    0) {
 		client_printf(client, "Setting socket reuse failed: %s\n",
 			      g_strerror(errno));
+		net_closesocket(fd);
 		return;
 	}
 	sa.sin_family = AF_INET;
@@ -547,6 +557,7 @@ static void client_create_new_server(Client * client, gchar * line)
 		if (found_used == TRUE) {
 			client_printf(client,
 				      "Starting server failed: no port available\n");
+			net_closesocket(fd);
 			return;
 		}
 	}
@@ -554,6 +565,7 @@ static void client_create_new_server(Client * client, gchar * line)
 	if (bind(fd, (struct sockaddr *) &sa, sizeof(sa)) < 0) {
 		client_printf(client, "Binding socket failed: %s\n",
 			      g_strerror(errno));
+		net_closesocket(fd);
 		return;
 	}
 	yes = sizeof(sa);
@@ -561,10 +573,13 @@ static void client_create_new_server(Client * client, gchar * line)
 		client_printf(client,
 			      "Getting socket address failed: %s\n",
 			      g_strerror(errno));
+		net_closesocket(fd);
 		return;
 	}
 	port = g_strdup_printf("%d", sa.sin_port);
 
+	net_closesocket(fd);
+
 	n = 0;
 	child_argv[n++] = g_strdup(console_server);
 	child_argv[n++] = g_strdup(console_server);
@@ -771,7 +786,6 @@ static void client_do_read(Client * client)
 {
 	int num;
 	int offset;
-	gboolean finished;
 
 	if (client->read_len == sizeof(client->read_buff)) {
 		/* We are in trouble now - something has gone
@@ -807,7 +821,6 @@ static void client_do_read(Client * client)
 
 	client->read_len += num;
 	offset = 0;
-	finished = FALSE;
 	while (offset < client->read_len) {
 		char *line = client->read_buff + offset;
 		int len = find_line(line, client->read_len - offset);
@@ -924,7 +937,7 @@ static void check_timeouts(void)
 
 static void select_loop(void)
 {
-	for (;;) {
+	while (!request_quit || client_list != NULL) {
 		fd_set read_res;
 		fd_set write_res;
 		int num;
@@ -1084,6 +1097,11 @@ static GOptionEntry commandline_entries[] = {
 	{NULL, '\0', 0, 0, NULL, NULL, NULL}
 };
 
+static void handle_usr1_signal(G_GNUC_UNUSED int signal)
+{
+	request_quit = TRUE;
+}
+
 int main(int argc, char *argv[])
 {
 	GOptionContext *context;
@@ -1102,6 +1120,7 @@ int main(int argc, char *argv[])
 	g_option_context_add_main_entries(context,
 					  commandline_entries, PACKAGE);
 	g_option_context_parse(context, &argc, &argv, &error);
+	g_option_context_free(context);
 	if (error != NULL) {
 		g_print("%s\n", error->message);
 		g_error_free(error);
@@ -1142,6 +1161,7 @@ int main(int argc, char *argv[])
 	game_list = load_game_types();
 	if (game_list) {
 		gchar *server_name;
+		g_list_foreach(game_list, (GFunc) g_free, NULL);
 		g_list_free(game_list);
 		server_name = g_find_program_in_path(get_server_path());
 		if (server_name) {
@@ -1158,9 +1178,23 @@ int main(int argc, char *argv[])
 	if (!setup_accept_sock(PIONEERS_DEFAULT_META_PORT))
 		return 1;
 
+	struct sigaction myusr1action;
+	struct sigaction oldusr1action;
+
+	myusr1action.sa_handler = handle_usr1_signal;
+	myusr1action.sa_flags = 0;
+	sigemptyset(&myusr1action.sa_mask);
+	sigaction(SIGUSR1, &myusr1action, &oldusr1action);
+
 	my_syslog(LOG_INFO, "Pioneers meta server started.");
 	select_loop();
 
+	sigaction(SIGUSR1, &oldusr1action, NULL);
+	g_free(pidfile);
+	g_free(redirect_location);
+	g_free(myhostname);
+	g_free(port_range);
+
 	net_finish();
 	return 0;
 }
