@@ -3,7 +3,7 @@
  *
  * Copyright (C) 1999 Dave Cole
  * Copyright (C) 2003-2007 Bas Wijnen <shevek@fmf.nl>
- * Copyright (C) 2005,2006 Roland Clobus <rclobus@bigfoot.com>
+ * Copyright (C) 2005-2010 Roland Clobus <rclobus@rclobus.nl>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,21 +33,31 @@ static gboolean mode_bad_version(Player * player, gint event);
 static gboolean mode_global(Player * player, gint event);
 static gboolean mode_unhandled(Player * player, gint event);
 static void player_setup(Player * player, int playernum,
-			 const gchar * name, gboolean force_viewer);
+			 const gchar * name, gboolean force_spectator);
 static Player *player_by_name(Game * game, char *name);
 
 #define tournament_minute 1000 * 60
+#define time_to_wait_for_players 30 * 60 * 1000
+
+/** Is the game a tournament game?
+ *  @param game The game
+ *  @return TRUE if this game is a tournament game
+*/
+static gboolean is_tournament_game(const Game * game)
+{
+	return game->params->tournament_time > 0;
+}
 
 /** Find a free number for a connecting player.
  * The number has not been used before.
  *  @param game The game
- *  @param force_viewer The connecting player must be a viewer
+ *  @param force_spectator The connecting player must be a spectator
  */
-static gint next_free_player_num(Game * game, gboolean force_viewer)
+static gint next_free_player_num(Game * game, gboolean force_spectator)
 {
 	gint idx;
 
-	if (!force_viewer) {
+	if (!force_spectator) {
 		GList *list;
 		gboolean player_taken[MAX_PLAYERS];
 		gint available = game->params->num_players;
@@ -58,7 +68,7 @@ static gint next_free_player_num(Game * game, gboolean force_viewer)
 		     list != NULL; list = g_list_next(list)) {
 			Player *player = list->data;
 			if (player->num >= 0
-			    && !player_is_viewer(game, player->num)) {
+			    && !player_is_spectator(game, player->num)) {
 				player_taken[player->num] = TRUE;
 				--available;
 			}
@@ -78,7 +88,7 @@ static gint next_free_player_num(Game * game, gboolean force_viewer)
 		}
 	}
 
-	/* No players available/wanted, look for a viewer number */
+	/* No players available/wanted, look for a spectator number */
 	idx = game->params->num_players;
 	while (player_by_num(game, idx) != NULL)
 		++idx;
@@ -102,7 +112,8 @@ static gboolean mode_global(Player * player, gint event)
 		if (player->devel != NULL)
 			deck_free(player->devel);
 		if (player->num >= 0
-		    && !player_is_viewer(game, player->num)) {
+		    && !player_is_spectator(game, player->num)
+		    && !player->disconnected) {
 			game->num_players--;
 			meta_report_num_players(game->num_players);
 		}
@@ -197,6 +208,11 @@ static gboolean tournament_start_cb(gpointer data)
 {
 	int i;
 	Game *game = (Game *) data;
+	GList *player;
+	gboolean human_player_present;
+
+	g_source_remove(game->tournament_timer);
+	game->tournament_timer = 0;
 
 	/* if game already started */
 	if (game->num_players == game->params->num_players)
@@ -210,12 +226,45 @@ static gboolean tournament_start_cb(gpointer data)
 				 ("The last player left, the "
 				  "tournament timer is reset."));
 		game->tournament_countdown = game->params->tournament_time;
+		return FALSE;
+	}
+
+	/* remove all disconnected players */
+	playerlist_inc_use_count(game);
+	for (player = game->player_list; player != NULL;
+	     player = g_list_next(player)) {
+		Player *p = player->data;
+		if (p->disconnected && !p->sm->use_cache) {
+			player_free(p);
+		}
+	}
+	playerlist_dec_use_count(game);
+
+	/* if no human players are present, quit */
+	playerlist_inc_use_count(game);
+	human_player_present = FALSE;
+	for (player = game->player_list;
+	     player != NULL && !human_player_present;
+	     player = g_list_next(player)) {
+		Player *p = player->data;
+		if (!player_is_spectator(game, p->num)
+		    && determine_player_type(p->style) == PLAYER_HUMAN) {
+			human_player_present = TRUE;
+		}
+	}
+	playerlist_dec_use_count(game);
+	if (!human_player_present) {
+		player_broadcast(player_none(game), PB_SILENT,
+				 FIRST_VERSION, LATEST_VERSION,
+				 "NOTE %s\n",
+				 N_("No human players present. Bye."));
+		request_server_stop(game);
+		return FALSE;
 	}
 
 	player_broadcast(player_none(game), PB_SILENT, FIRST_VERSION,
 			 LATEST_VERSION, "NOTE %s\n",
 			 N_("Game starts, adding computer players."));
-	game->tournament_timer = 0;
 
 	/* add computer players to start game */
 	for (i = game->num_players; i < game->params->num_players; i++) {
@@ -337,8 +386,10 @@ gchar *player_new_computer_player(Game * game)
 	Player *player;
 	gchar *name;
 
+	/* Reserve the name, so the names of the computer players will
+	   be unique */
 	name = generate_name_for_computer_player(game);
-	player = player_new(game, name, "square");
+	player = player_new(game, name);
 	player->disconnected = TRUE;
 	sm_goto(player->sm, (StateFunc) mode_idle);
 	return name;
@@ -347,7 +398,7 @@ gchar *player_new_computer_player(Game * game)
 /** Allocate a new Player struct.
  *  The StateMachine is not initialized.
  *   */
-Player *player_new(Game * game, const gchar * name, const gchar * style)
+Player *player_new(Game * game, const gchar * name)
 {
 	Player *player;
 	StateMachine *sm;
@@ -371,7 +422,7 @@ Player *player_new(Game * game, const gchar * name, const gchar * style)
 	player->islands_discovered = 0;
 	player->disconnected = FALSE;
 	player->name = g_strdup(name);
-	player->style = g_strdup(style);
+	player->style = NULL;
 	player->special_points = NULL;
 	player->special_points_next_id = 0;
 
@@ -421,7 +472,7 @@ Player *player_new_connection(Game * game, int fd, const gchar * location)
 		return NULL;
 	}
 
-	player = player_new(game, name, "square");
+	player = player_new(game, name);
 	sm = player->sm;
 	sm_use_fd(sm, fd, TRUE);
 	g_free(player->location);
@@ -475,7 +526,7 @@ static void player_set_name_real(Player * player, gchar * name,
 }
 
 static void player_setup(Player * player, int playernum,
-			 const gchar * name, gboolean force_viewer)
+			 const gchar * name, gboolean force_spectator)
 {
 	gchar nm[MAX_NAME_LENGTH + 1];
 	Game *game = player->game;
@@ -484,10 +535,10 @@ static void player_setup(Player * player, int playernum,
 
 	player->num = playernum;
 	if (player->num < 0) {
-		player->num = next_free_player_num(game, force_viewer);
+		player->num = next_free_player_num(game, force_spectator);
 	}
 
-	if (!player_is_viewer(game, player->num)) {
+	if (!player_is_spectator(game, player->num)) {
 		game->num_players++;
 		meta_report_num_players(game->num_players);
 	}
@@ -500,10 +551,10 @@ static void player_setup(Player * player, int playernum,
 
 	/* give the player her new name */
 	if (name == NULL) {
-		if (player_is_viewer(game, player->num)) {
+		if (player_is_spectator(game, player->num)) {
 			gint num = 1;
 			do {
-				sprintf(nm, _("Viewer %d"), num++);
+				sprintf(nm, _("Spectator %d"), num++);
 			} while (player_by_name(game, nm) != NULL);
 		} else {
 			sprintf(nm, _("Player %d"), player->num);
@@ -570,13 +621,32 @@ void player_free(Player * player)
 	sm_free(player->sm);
 }
 
+static gboolean timed_out(gpointer data)
+{
+	Game *game = data;
+	log_message(MSG_INFO,
+		    _(""
+		      "Was hanging around for too long without players... bye.\n"));
+	player_broadcast(player_none(game), PB_SILENT,
+			 FIRST_VERSION, LATEST_VERSION,
+			 "NOTE %s\n",
+			 N_("No human players present. Bye."));
+	request_server_stop(game);
+	return FALSE;
+}
+
 void player_archive(Player * player)
 {
 	StateFunc state;
 	Game *game = player->game;
 
-	/* If this was a viewer, forget about him */
-	if (player_is_viewer(game, player->num)) {
+	/* If this was a spectator, forget about him */
+	if (player_is_spectator(game, player->num)) {
+		player_free(player);
+		return;
+	}
+	/* If this game can't be started, forget old players */
+	if (game_is_unstartable(game)) {
 		player_free(player);
 		return;
 	}
@@ -615,6 +685,32 @@ void player_archive(Player * player)
 	player->disconnected = TRUE;
 	game->num_players--;
 	meta_report_num_players(game->num_players);
+
+	/* if no human players are present, start timer */
+	playerlist_inc_use_count(game);
+	gboolean human_player_present = FALSE;
+	GList *pl = game->player_list;
+	for (pl = game->player_list;
+	     pl != NULL && !human_player_present; pl = g_list_next(pl)) {
+		Player *p = pl->data;
+		if (!player_is_spectator(game, p->num)
+		    && !p->disconnected
+		    && determine_player_type(p->style) == PLAYER_HUMAN) {
+			human_player_present = TRUE;
+		}
+	}
+	playerlist_dec_use_count(game);
+	if (!human_player_present && game->no_humans_timer == 0
+	    && is_tournament_game(game)) {
+		game->no_humans_timer =
+		    g_timeout_add(time_to_wait_for_players, timed_out,
+				  game);
+		player_broadcast(player_none(game), PB_SILENT,
+				 FIRST_VERSION, LATEST_VERSION,
+				 "NOTE %s\n",
+				 N_
+				 ("The last human player left. Waiting for the return of a player."));
+	}
 }
 
 /* Try to revive the player
@@ -629,6 +725,14 @@ void player_revive(Player * newp, char *name)
 	gboolean reviving_player_in_setup;
 	gchar *safe_name;
 
+	if (game->no_humans_timer != 0) {
+		g_source_remove(game->no_humans_timer);
+		game->no_humans_timer = 0;
+		player_broadcast(player_none(game), PB_SILENT,
+				 FIRST_VERSION, LATEST_VERSION,
+				 "NOTE %s\n", N_("Resuming the game."));
+	}
+
 	/* first see if a player with the given name exists */
 	if (name) {
 		playerlist_inc_use_count(game);
@@ -763,14 +867,14 @@ void player_revive(Player * newp, char *name)
 	return;
 }
 
-gboolean mode_viewer(Player * player, gint event)
+gboolean mode_spectator(Player * player, gint event)
 {
 	gint num;
 	Game *game = player->game;
 	StateMachine *sm = player->sm;
 	Player *other;
 
-	sm_state_name(sm, "mode_viewer");
+	sm_state_name(sm, "mode_spectator");
 	if (event != SM_RECV)
 		return FALSE;
 	/* first see if this is a valid event for this mode */
@@ -799,7 +903,7 @@ gboolean mode_viewer(Player * player, gint event)
 	if (other == NULL) {
 		player_send(player, FIRST_VERSION, LATEST_VERSION, "Ok\n");
 		player_broadcast(player, PB_ALL, FIRST_VERSION,
-				 LATEST_VERSION, "was viewer %d\n",
+				 LATEST_VERSION, "was spectator %d\n",
 				 player->num);
 		sm_set_use_cache(player->sm, TRUE);
 		player_setup(player, -1, player->name, FALSE);
@@ -908,7 +1012,7 @@ static void start_tournament_mode(Player * player)
 {
 	Game *game = player->game;
 
-	if (game->params->tournament_time > 0) {
+	if (is_tournament_game(game)) {
 		/* if first player in and this is a tournament start the timer */
 		if (game->num_players == 1) {
 			game->tournament_countdown =
@@ -920,7 +1024,9 @@ static void start_tournament_mode(Player * player)
 			g_timeout_add(1000, &talk_about_tournament_cb,
 				      game);
 		} else {
-			if (game->tournament_timer != 0) {
+			if (game->tournament_timer != 0
+			    && game->num_players !=
+			    game->params->num_players) {
 				player_send(player, FIRST_VERSION,
 					    LATEST_VERSION, "NOTE %s\n",
 					    N_
@@ -1057,7 +1163,7 @@ Player *player_by_num(Game * game, gint num)
 	return NULL;
 }
 
-gboolean player_is_viewer(Game * game, gint player_num)
+gboolean player_is_spectator(Game * game, gint player_num)
 {
 	return game->params->num_players <= player_num;
 }
@@ -1074,7 +1180,7 @@ Player *player_none(Game * game)
 	return &player;
 }
 
-/** Broadcast a message to all players and viewers - prepend "player %d " to
+/** Broadcast a message to all players and spectators - prepend "player %d " to
  * all players except the one generating the message.
  * Also prepend 'extension' when this message is a protocol extension.
  *
@@ -1144,7 +1250,7 @@ void player_broadcast_extension(Player * player, BroadcastType type,
 	va_list ap;
 
 	va_start(ap, fmt);
-	buff = sm_vformat(fmt, ap);
+	buff = game_vprintf(fmt, ap);
 	va_end(ap);
 
 	player_broadcast_internal(player, type, buff, TRUE,
@@ -1153,7 +1259,7 @@ void player_broadcast_extension(Player * player, BroadcastType type,
 	g_free(buff);
 }
 
-/** Broadcast a message to all players and viewers */
+/** Broadcast a message to all players and spectators */
 void player_broadcast(Player * player, BroadcastType type,
 		      ClientVersionType first_supported_version,
 		      ClientVersionType last_supported_version,
@@ -1163,7 +1269,7 @@ void player_broadcast(Player * player, BroadcastType type,
 	va_list ap;
 
 	va_start(ap, fmt);
-	buff = sm_vformat(fmt, ap);
+	buff = game_vprintf(fmt, ap);
 	va_end(ap);
 
 	player_broadcast_internal(player, type, buff, FALSE,
@@ -1186,7 +1292,7 @@ void player_send(Player * player,
 		return;
 
 	va_start(ap, fmt);
-	buff = sm_vformat(fmt, ap);
+	buff = game_vprintf(fmt, ap);
 	va_end(ap);
 
 	sm_write(player->sm, buff);
@@ -1207,7 +1313,7 @@ void player_send_uncached(Player * player,
 		return;
 
 	va_start(ap, fmt);
-	buff = sm_vformat(fmt, ap);
+	buff = game_vprintf(fmt, ap);
 	va_end(ap);
 
 	sm_write_uncached(player->sm, buff);
