@@ -254,9 +254,7 @@ void next_setup_player(Game * game)
 	} else {
 		/* First setup phase
 		 */
-		Player *player;
 		game->setup_player = player_next_real(game->setup_player);
-		player = game->setup_player->data;
 		/* Last player gets double setup
 		 */
 		game->double_setup
@@ -331,6 +329,11 @@ static void try_start_game(Game * game)
 	if (num != game->params->num_players)
 		return;
 
+	if (game_is_unstartable(game)) {
+		/* this game cannot be started, don't enter the setup phase */
+		return;
+	}
+
 	if (numturn > 0) {
 		/* someone got disconnected. Now everyone's back. Let's
 		   continue the game... */
@@ -340,6 +343,10 @@ static void try_start_game(Game * game)
 
 	/* All players have connected, and are ready to begin
 	 */
+	if (game->tournament_timer != 0) {
+		g_source_remove(game->tournament_timer);
+		game->tournament_timer = 0;
+	}
 	meta_start_game();
 	game->setup_player = player_first_real(game);
 	while (((Player *) game->setup_player->data)->num < 0)
@@ -509,7 +516,7 @@ gboolean mode_pre_game(Player * player, gint event)
 		/* Tell the player that he exists.  This is not done in
 		 * player_set_name, because at that point the client doesn't
 		 * know how many players are in the game, and therefore if
-		 * he is a player or a viewer. */
+		 * he is a player or a spectator. */
 		/* Tell the other players about this player */
 		player_broadcast(player, PB_OTHERS, FIRST_VERSION,
 				 LATEST_VERSION, "is %s\n", player->name);
@@ -546,6 +553,8 @@ gboolean mode_pre_game(Player * player, gint event)
 			return TRUE;
 		}
 		if (sm_recv(sm, "gameinfo")) {
+			GList *list;
+
 			player_send_uncached(player, FIRST_VERSION,
 					     LATEST_VERSION, "gameinfo\n");
 			map_traverse_const(map, send_gameinfo_uncached,
@@ -665,81 +674,70 @@ gboolean mode_pre_game(Player * player, gint event)
 					     "development-bought %d\n",
 					     game->develop_next);
 
-			/* send player info about what he has:
+			/* Send player info about what he has:
 			   resources, dev cards, roads, # roads,
 			   # bridges, # ships, # settles, # cities,
 			   # soldiers, road len, dev points,
-			   who has longest road/army */
-			/* Only send this when the player is not a viewer */
-			if (!player_is_viewer(game, player->num)) {
-				GList *list;
-
-				player_send_uncached(player, FIRST_VERSION,
+			   who has longest road/army,
+			   spectators will receive an empty list */
+			player_send_uncached(player, FIRST_VERSION,
+					     LATEST_VERSION,
+					     "playerinfo: resources: %R\n",
+					     player->assets);
+			player_send_uncached(player, FIRST_VERSION,
+					     LATEST_VERSION,
+					     "playerinfo: numdevcards: %d\n",
+					     player->devel->num_cards);
+			for (i = 0; i < player->devel->num_cards; i++) {
+				player_send_uncached(player,
+						     FIRST_VERSION,
 						     LATEST_VERSION,
-						     "playerinfo: resources: %R\n",
-						     player->assets);
-				player_send_uncached(player, FIRST_VERSION,
-						     LATEST_VERSION,
-						     "playerinfo: numdevcards: %d\n",
-						     player->devel->
-						     num_cards);
-				for (i = 0; i < player->devel->num_cards;
-				     i++) {
-					player_send_uncached(player,
-							     FIRST_VERSION,
-							     LATEST_VERSION,
-							     "playerinfo: devcard: %d %d\n",
-							     (gint)
-							     player->
-							     devel->
-							     cards[i].type,
-							     player->
-							     devel->
-							     cards[i].
-							     turn_bought);
-				}
-				player_send_uncached(player, FIRST_VERSION,
-						     LATEST_VERSION,
-						     "playerinfo: %d %d %d %d %d %d %d %d %d %d %d %d %d %d\n",
-						     player->num_roads,
-						     player->num_bridges,
-						     player->num_ships,
+						     "playerinfo: devcard: %d %d\n",
+						     (gint) player->
+						     devel->cards[i].type,
 						     player->
-						     num_settlements,
-						     player->num_cities,
-						     player->num_soldiers,
-						     player->road_len,
-						     player->chapel_played,
-						     player->univ_played,
-						     player->gov_played,
-						     player->libr_played,
-						     player->market_played,
-						     (player->num ==
-						      longestroadpnum),
-						     (player->num ==
-						      largestarmypnum));
-
-				/* Send special points */
-				list = player->special_points;
-				while (list) {
-					Points *points = list->data;
-					player_send_uncached(player,
-							     FIRST_VERSION,
-							     LATEST_VERSION,
-							     "get-point %d %d %d %s\n",
-							     player->num,
-							     points->id,
-							     points->
-							     points,
-							     points->name);
-					list = g_list_next(list);
-				}
+						     devel->cards[i].
+						     turn_bought);
 			}
-			/* send info about other players */
+			player_send_uncached(player, FIRST_VERSION,
+					     LATEST_VERSION,
+					     "playerinfo: %d %d %d %d %d %d %d %d %d %d %d %d %d %d\n",
+					     player->num_roads,
+					     player->num_bridges,
+					     player->num_ships,
+					     player->num_settlements,
+					     player->num_cities,
+					     player->num_soldiers,
+					     player->road_len,
+					     player->chapel_played,
+					     player->univ_played,
+					     player->gov_played,
+					     player->libr_played,
+					     player->market_played,
+					     (player->num ==
+					      longestroadpnum),
+					     (player->num ==
+					      largestarmypnum));
+
+			/* Send special points */
+			list = player->special_points;
+			while (list) {
+				Points *points = list->data;
+				player_send_uncached(player,
+						     FIRST_VERSION,
+						     LATEST_VERSION,
+						     "get-point %d %d %d %s\n",
+						     player->num,
+						     points->id,
+						     points->points,
+						     points->name);
+				list = g_list_next(list);
+			}
+
+			/* Send info about other players */
 			for (next = player_first_real(game); next != NULL;
 			     next = player_next_real(next)) {
 				Player *p = (Player *) next->data;
-				GList *list;
 				gint numassets = 0;
 				if (p->num == player->num)
 					continue;
@@ -779,7 +777,7 @@ gboolean mode_pre_game(Player * player, gint event)
 				}
 			}
 
-			/* send build info for the current player
+			/* Send build info for the current player
 			   - what builds the player was in the process
 			   of building when he disconnected */
 			for (next = player->build_list;
@@ -917,16 +915,17 @@ gboolean mode_pre_game(Player * player, gint event)
 				if (!sm_is_connected(sm))
 					/* This happens when the connection is
 					 * dropped when the cache is sent */
-					sm_goto(sm,
-						(StateFunc) mode_viewer);
+					sm_goto(sm, (StateFunc)
+						mode_spectator);
 				else
 					sm_pop(sm);
 			} else {
-				if (!player_is_viewer(game, player->num))
+				if (!player_is_spectator
+				    (game, player->num))
 					sm_goto(sm, (StateFunc) mode_idle);
 				else
-					sm_goto(sm,
-						(StateFunc) mode_viewer);
+					sm_goto(sm, (StateFunc)
+						mode_spectator);
 			}
 			try_start_game(game);
 			return TRUE;
