@@ -238,7 +238,10 @@ Game *server_start(const GameParams * params, const gchar * hostname,
 	game = game_new(params);
 	g_assert(game->server_port == NULL);
 	game->server_port = g_strdup(port);
-	game->hostname = g_strdup(hostname);
+	g_assert(game->hostname == NULL);
+	if (hostname && strlen(hostname) > 0) {
+		game->hostname = g_strdup(hostname);
+	}
 	game->random_order = random_order;
 	if (!game_server_start(game, register_server, meta_server_name)) {
 		game_free(game);
@@ -585,3 +588,8 @@ void server_init(void)
 	sigaction(SIGPIPE, &sa, NULL);
 #endif				/* G_OS_WIN32 */
 }
+
+gboolean game_is_unstartable(Game * game)
+{
+	return game->params->num_build_type[BUILD_SETTLEMENT] < 2;
+}
