@@ -130,6 +130,8 @@ struct Game {
 
 	gint no_player_timeout;	/* time to wait for players */
 	guint no_player_timer;	/* glib timer identifier */
+
+	guint no_humans_timer;	/* timer id: no human players are present */
 };
 
 /**** global variables ****/
@@ -172,7 +174,7 @@ typedef enum {
 	PB_OTHERS
 } BroadcastType;
 gchar *player_new_computer_player(Game * game);
-Player *player_new(Game * game, const gchar * name, const gchar * style);
+Player *player_new(Game * game, const gchar * name);
 Player *player_new_connection(Game * game, int fd, const gchar * location);
 Player *player_by_num(Game * game, gint num);
 void player_set_name(Player * player, gchar * name);
@@ -201,10 +203,10 @@ GList *player_first_real(Game * game);
 GList *player_next_real(GList * last);
 GList *list_from_player(Player * player);
 GList *next_player_loop(GList * current, Player * first);
-gboolean mode_viewer(Player * player, gint event);
+gboolean mode_spectator(Player * player, gint event);
 void playerlist_inc_use_count(Game * game);
 void playerlist_dec_use_count(Game * game);
-gboolean player_is_viewer(Game * game, gint player_num);
+gboolean player_is_spectator(Game * game, gint player_num);
 
 /* pregame.c */
 gboolean mode_pre_game(Player * player, gint event);
@@ -243,6 +245,7 @@ Game *server_start(const GameParams * params, const gchar * hostname,
 gboolean server_stop(Game * game);
 gboolean server_is_running(Game * game);
 gint accept_connection(gint in_fd, gchar ** location);
+gboolean game_is_unstartable(Game * game);
 
 /**** game list control functions ****/
 void game_list_prepare(void);
