@@ -251,7 +251,7 @@ static void build_add(Player * player, BuildType type, gint x, gint y,
 				    (player->special_points_next_id++,
 				     first_island ?
 				     N_("Island Discovery Bonus")
-				     : N_("" "Additional Island Bonus"),
+				     : N_("Additional Island Bonus"),
 				     points);
 			player->islands_discovered++;
 		}
@@ -417,8 +417,6 @@ gboolean check_victory(Player * player)
 	}
 
 	if (points >= game->params->victory_points) {
-		GList *list;
-
 		player_broadcast(player, PB_ALL, FIRST_VERSION,
 				 LATEST_VERSION, "won with %d\n", points);
 		game->is_game_over = TRUE;
